//==============================================================================
//Copyright(c), Meilhaus Electronic GmbH
//              http://www.meilhaus.de
//              support@meilhaus.de
//
//Source File : uMain.pas
//Description : Example program for writing to Frequency-out (FO) sub-devices
//              - Open main driver (ME-iDS)
//              - Go through the list of all devices and find FO subdevices
//              - Add FO devices to the combobox, so the user can select one
//              - First set the configuration of the selected subdevice
//              - Write data to selected subdevice when user clicks on button "Write Data"
//              - Written values can be read back by click on "Read Data"
//              - Close main driver (ME-iDS) when program is closed
//==============================================================================
//  To emit a digital signal two values of the FO subdevice must be set:
//  - 1. "Ticks total" = value of the entire period of the signal
//  - 2. "Ticks first phase" = value of the first phase of the signal (first phase can be high or low)
//    Therefore the function meIOSingle() must be called twice using different flags
//
//  digital signal:
//      +-------------------+         +---
//      |                   |         |
//      |                   |         |
//  ----+                   +---------+
//
//      ^-------Ticks total-----------^
//
//      ^--Ticks 1st phase--^
//
//
//  How to start several (a group of) FO-subdevices (FO-channels) synchronously:
//  - configure each subdevice with the flag 'ME_TRIG_CHAN_SYNCHRONOUS'
//  - write the values for 'Ticks total' and 'Ticks first phase' to each channel without the flag 'Synchronous',
//  - set the flag 'ME_TRIG_CHAN_SYNCHRONOUS' for the last subdevice in the group.
//  Port configuration:
//  - Configuration of the output driver of devices with sink, source, high impedance capabilities (e.g. ME-5810A, ME-5002)
//  Configuration:
//    Immediate output = start the output of the configured subdevice(s) or channel(s)
//    after values "Ticks total" and "Ticks first phase" are set
//    Synchronous output = don't start the selected subdevice or channel, set only values for "Ticks total" and "Ticks first phase"
//  - (Also see ME-iDS Help section: Supported Hardware / ME-46xx Description / Special features / Analog output subdevices)
//  Read:
//  - read back the values of the previously emitted signal
//    Ticks total: the value of the entire period
//    Ticks first phase: the value of the first phase of the period
//  Write:
//  - Signal start level: Start signal with high or low phase
//    High: first phase of the signal will be high
//    Low: first phase of the signal will he low
//  - Signal start type: start immediately when button clicked or at the end of the currently emitted signal period
//    Hard: start the output of the signal immediately (on button click)
//    Soft: Controlled start of the new signal when the period of the current signal (if any) is over
//  - Write mode:
//    Direct: This subdevice or channel will be started alone
//    Synchronous: All those subdevices which have been configured with the 'iTrigChan' = ME_TRIG_CHAN_SYNCHRONOUS
//    in meIOSingleConfig will be started simultaneously together with this subdevice
//------------------------------------------------------------------------------
unit uMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ComCtrls, meIDS, meError;

type
//-------------------------------------------------
{TFODevice}
  TFoSubdevice = class(TCollectionItem)
  private
    FDeviceName: string;
    FIndexDevice: Integer;
    FIndexSubdevice: Integer;
    FNumberOfChannels: Integer;
    FSubdeviceCaps: Integer;
  published
    property DeviceName: string read FDeviceName;
    property IndexDevice: Integer read FIndexDevice;
    property IndexSubdevice: Integer read FIndexSubdevice;
    property NumberOfChannels: Integer read FNumberOfChannels;
    property SubdeviceCaps: Integer read FSubdeviceCaps;
  end;

  TFoSubdevices = class(TCollection)
  private
    function GetSubdevice(Index: Integer): TFoSubdevice;
  public
    function AddSubdevice(ADeviceName: string; AIdxDevice: Integer; AIdxSubdevice: Integer; ANumOfChannels: Integer; ASubdeviceCaps: Integer): TFoSubdevice;
    property Items[Index: Integer]: TFoSubdevice read GetSubdevice; default;
    Constructor Create;
  end;
//--End--------------------------------------------

  TfrmMain = class(TForm)
    btnClearHwInfo: TButton;
    btnAbout: TButton;
    lbl1: TLabel;
    memoInfo: TMemo;
    btnClose: TButton;
    grpBox1: TGroupBox;
    lbl3: TLabel;
    coBoxSubDevs: TComboBox;
    btnSetConfig: TButton;
    btnReadBack: TButton;
    btnReset: TButton;
    btnDeviceWrite: TButton;
    rgStartLevel: TRadioGroup;
    rgStartType: TRadioGroup;
    rgWriteMode: TRadioGroup;
    editFirstPhase: TEdit;
    lbl2: TLabel;
    lbl4: TLabel;
    editTotalTime: TEdit;
    lbl5: TLabel;
    coBoxChannels: TComboBox;
    grpBoxConfig: TGroupBox;
    rBtnImmediateOutput: TRadioButton;
    rBtnSynchronousOutput: TRadioButton;
    grpBoxWrite: TGroupBox;
    coBoxPortConfiguration: TComboBox;
    lbl6: TLabel;
    procedure btnAboutClick(Sender: TObject);
    procedure btnClearHwInfoClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnResetClick(Sender: TObject);
    procedure btnSetConfigClick(Sender: TObject);
    procedure btnReadBackClick(Sender: TObject);
    procedure btnDeviceWriteClick(Sender: TObject);
    procedure coBoxSubDevsChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private-Deklarationen }
    openErrSuccess: Integer;
    lineNumber: Integer;
    foSubdevices: TFoSubdevices;
    function MeVersionToString(version: Integer): string;
    function GetSubDevTypeName(subdeviceType: Integer): string;
    procedure GetFoSubdevices(numDevices: Integer);
    procedure GetSubdevicesByType(idxDevice: Integer; subdeviceType: Integer);
    procedure OpenMEiDSDriver();
    procedure CloseMEiDSDriver();
    procedure DisplayVersionLibrary();
    procedure DisplayVersionMainDriver();
    procedure DisplaySubdevices();
    function GetDevicePlugged(idxDevice: Integer; var plugged: Integer): Integer;
    procedure DisplayErrorMessage(ErrMsg: string; meError: Integer);
    procedure FillPortConfiguration;
    procedure AddFoSubdevice(devName: string; idxDevice: Integer; idxSubdevice: Integer; subdeviceType: Integer);
    function GetSingleConfigParam(foSubdevice: TFoSubdevice): Integer;
    function GetOutputParam(): Integer;
    function ReadSingleValue(foSubdevice: TFoSubdevice; idxChannel: Integer;
      flags: Integer; var value: Integer): Integer;
    function WriteSingleValue(foSubdevice: TFoSubdevice; idxChannel: Integer;
      flags: Integer; value: Integer): Integer;
    procedure DisplayReadBackValues(foSubdevice: TFoSubdevice;
      idxChannel: Integer; value1stPhase: Integer; valueTotal: Integer);
    function ConfigureFoSubdevice(foSubdevice: TFoSubdevice; idxChannel: Integer): Integer;
    function GetFoConfigText: string;
    function GetFoTotalTimeFlags(): Integer;
    function GetFoFirstPhaseTimeFlags(): Integer;
    function ConvertTimeToTicks(foSubdevice: TFoSubdevice; timer: Integer;
      var timeInSeconds: Double; var ticks: Integer): Integer;
    procedure DisplayTimeValues(foSubdevice: TFoSubdevice; idxChannel: Integer;
      secondsAchievedTotal, secondsAchievedFirstPhase: Double;
      ticksTotal, ticksFirstPhase: Integer);
    procedure FillChannels(foSubdevice: TFoSubdevice);

  public
    { Public-Deklarationen }
  end;

var
  frmMain: TfrmMain;

implementation

uses uAbout;

{$R *.dfm}

const
  ME_IDS_NOTOPENED = -1;


//----------------------------------------------
{TFODevice}
Constructor TFoSubdevices.Create();
begin
  inherited Create(TFoSubdevice);
end;

function TFoSubdevices.GetSubdevice(Index: Integer): TFoSubdevice;
begin
  Result := TFoSubdevice(inherited GetItem(Index));
end;

function TFoSubdevices.AddSubdevice(ADeviceName: string; AIdxDevice: Integer; AIdxSubdevice: Integer; ANumOfChannels: Integer; ASubdeviceCaps: Integer): TFoSubdevice;
begin
  Result := inherited Add as TFoSubdevice;
  Result.FDeviceName := ADeviceName;
  Result.FIndexDevice := AIdxDevice;
  Result.FIndexSubdevice := AIdxSubdevice;
  Result.FNumberOfChannels := ANumOfChannels;
  Result.FSubdeviceCaps := ASubdeviceCaps;
end;
//----------------------------------------------
{TfrmMain}
procedure TfrmMain.FormCreate(Sender: TObject);
begin
  openErrSuccess := ME_IDS_NOTOPENED;
  lineNumber := 0;
  foSubdevices := TFoSubdevices.Create;
  FillPortConfiguration;

  Application.HintHidePause := 5000;
end;

procedure TfrmMain.FormShow(Sender: TObject);
begin
  editFirstPhase.Text := FloatToStr(0.2);
  OpenMEiDSDriver();
end;

procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  CloseMEiDSDriver();
  foSubdevices.Free;
end;

procedure TfrmMain.OpenMEiDSDriver();
var
  numDevices: Integer;
begin
  memoInfo.Lines.Append('Opening Main driver...');
  openErrSuccess := meOpen(ME_OPEN_NO_FLAGS);

  if openErrSuccess = ME_ERRNO_SUCCESS then
  begin
    DisplayVersionLibrary;
    DisplayVersionMainDriver;
    memoInfo.Lines.Append('');

    meQueryNumberDevices(numDevices);
    memoInfo.Lines.Append('Number of devices found: ' + IntToStr(numDevices));
    GetFoSubdevices(numDevices);
  end else
  begin
    DisplayErrorMessage('Error opening Main driver. Error: ', openErrSuccess);
  end;
end;

procedure TfrmMain.GetFoSubdevices(numDevices: Integer);
var
  meError: Integer;
  idxDevice: Integer;
  plugged: Integer;
begin
  if (openErrSuccess = ME_ERRNO_SUCCESS) and (numDevices > 0) then
  begin
    memoInfo.Lines.Add('Retrieving sub-devices of type ME_TYPE_FO (Frequency Out)...');
    for idxDevice := 0 to numDevices - 1 do
    begin
      meError := GetDevicePlugged(idxDevice, plugged);

      if (meError = ME_ERRNO_SUCCESS) and (plugged = ME_PLUGGED_IN) then
      begin
        GetSubdevicesByType(idxDevice, ME_TYPE_FO);
      end else
      begin
        memoInfo.Lines.Append('Device ' + IntToStr(idxDevice) + ' not plugged in');
      end;
    end;
  end;
  DisplaySubdevices();
end;


procedure TfrmMain.DisplaySubdevices();
begin
  if coBoxSubDevs.Items.Count > 0 then
  begin
    memoInfo.Lines.Append('Number of FO sub-devices found: ' + IntToStr(coBoxSubDevs.Items.Count));
    memoInfo.Lines.AddStrings(coBoxSubDevs.Items);
    memoInfo.Lines.Add('');
    coBoxSubDevs.ItemIndex := 0;
    coBoxSubDevsChange(coBoxSubDevs);
    btnReadBack.Enabled := True;
    btnSetConfig.Enabled := True;
    btnDeviceWrite.Enabled := True;
    btnReset.Enabled := True;
  end else
  begin
    memoInfo.Lines.Append('No FO sub-devices found.');
    CloseMEiDSDriver;
  end;
end;

procedure TfrmMain.GetSubdevicesByType(idxDevice: Integer; subdeviceType: Integer);
var
  idxSubdevice: Integer;
  numSubdevices: Integer;
  meError: Integer;
  idxMatchedSubdevice: Integer;
  devName: array[0..ME_DEVICE_NAME_MAX_COUNT] of AnsiChar;
begin
  meError := meQueryNumberSubdevices(idxDevice, numSubdevices);
  if meError = ME_ERRNO_SUCCESS then
  begin
    meQueryNameDevice(idxDevice, devName, ME_DEVICE_NAME_MAX_COUNT);
    idxSubdevice := 0;
    while idxSubdevice < numSubdevices do
    begin
      meError := meQuerySubdeviceByType(idxDevice,
                                        idxSubdevice,
                                        subdeviceType,
                                        ME_SUBTYPE_ANY,
                                        idxMatchedSubdevice);

      if meError = ME_ERRNO_SUCCESS then
      begin
        AddFoSubdevice(string(devName), idxDevice, idxMatchedSubdevice, subdeviceType);
        idxSubdevice := idxMatchedSubdevice + 1;
      end else
      begin
        Break;
      end;
    end;
  end;
end;

procedure TfrmMain.AddFoSubdevice(devName: string; idxDevice: Integer; idxSubdevice: Integer; subdeviceType: Integer);
var
  numChannels: Integer;
  subdevCaps: Integer;
begin
  meQueryNumberChannels(idxDevice,
                        idxSubdevice,
                        numChannels);
  meQuerySubdeviceCaps(idxDevice, idxSubdevice, subdevCaps);
  foSubdevices.AddSubdevice(devName, idxDevice, idxSubdevice, numChannels, subdevCaps);

  coBoxSubDevs.Items.Append(devName
                            + ',  Device ' + IntToStr(idxDevice)
                            + ',  Subdevice ' + IntToStr(idxSubdevice)
                            + ',  Number of channels ' + IntToStr(numChannels)
                            + ',  Type: ' + GetSubDevTypeName(subdeviceType));
end;

function TfrmMain.GetSingleConfigParam(foSubdevice: TFoSubdevice): Integer;
begin
  if (foSubdevice.SubdeviceCaps and ME_CAPS_FIO_SINK_SOURCE) = ME_CAPS_FIO_SINK_SOURCE then
  begin
    Result := Integer(coBoxPortConfiguration.Items.Objects[coBoxPortConfiguration.ItemIndex]);
  end else
  begin
    Result := ME_SINGLE_CONFIG_FIO_OUTPUT;
  end;
end;

function TfrmMain.GetOutputParam(): Integer;
begin
  if rBtnImmediateOutput.Checked then
  begin
    Result := ME_TRIG_CHAN_DEFAULT;
  end else
  begin
    Result := ME_TRIG_CHAN_SYNCHRONOUS;
  end;
end;

procedure TfrmMain.btnCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmMain.btnSetConfigClick(Sender: TObject);
var
  meError: Integer;
  idxChannel: Integer;
  foSubdevice: TFoSubdevice;
begin
  if coBoxSubDevs.ItemIndex >= 0 then
  begin
    foSubdevice := foSubdevices.Items[coBoxSubDevs.ItemIndex];
    idxChannel := coBoxChannels.ItemIndex;
    meError := ConfigureFoSubdevice(foSubdevice, idxChannel);

    if meError = ME_ERRNO_SUCCESS then
    begin
      memoInfo.Lines.Add('Set Config: Device ' + IntToStr(foSubdevice.IndexDevice)
                         + ', Subdevice ' + IntToStr(foSubdevice.IndexSubdevice)
                         + ', Channel ' + IntToStr(idxChannel)
                         + ', ' + GetFoConfigText);
    end else
    begin
      DisplayErrorMessage('meIOSingleConfig - Error: ', meError);
    end;
  end;
end;

function TfrmMain.GetFoConfigText: string;
begin
  if rBtnImmediateOutput.Checked then
  begin
    Result := rBtnImmediateOutput.Caption;
  end else
  begin
    Result := rBtnSynchronousOutput.Caption;
  end;
end;

procedure TfrmMain.btnReadBackClick(Sender: TObject);
var
  meError: Integer;
  idxChannel: Integer;
  foSubdevice: TFoSubdevice;
  valueTotal: Integer;
  value1stPhase: Integer;
begin
  if (coBoxSubDevs.ItemIndex >= 0) then
  begin
    foSubdevice := foSubdevices.Items[coBoxSubDevs.ItemIndex];
    idxChannel := coBoxChannels.ItemIndex;
    memoInfo.Lines.Add(#13#10'Read data');

    meError := ReadSingleValue(foSubdevice, idxChannel, ME_IO_SINGLE_TYPE_FIO_TICKS_TOTAL, valueTotal);
    if meError = ME_ERRNO_SUCCESS then
    begin
      meError := ReadSingleValue(foSubdevice, idxChannel, ME_IO_SINGLE_TYPE_FIO_TICKS_FIRST_PHASE, value1stPhase);

      if meError = ME_ERRNO_SUCCESS then
      begin
        DisplayReadBackValues(foSubdevice, idxChannel, value1stPhase, valueTotal);
      end;
    end;
    if meError <> ME_ERRNO_SUCCESS then
    begin
      DisplayErrorMessage('Error: ', meError);
    end;
  end;
end;

function TfrmMain.ReadSingleValue(foSubdevice: TFoSubdevice; idxChannel: Integer;
  flags: Integer; var value: Integer): Integer;
var
  meError: Integer;
  io_single: array[0..0] of TMeIOSingle;
begin
  io_single[0].iDevice := foSubdevice.IndexDevice;
  io_single[0].iSubDevice := foSubdevice.IndexSubdevice;
  io_single[0].iChannel := idxChannel;
  io_single[0].iDir := ME_DIR_INPUT;
  io_single[0].iValue := 0;
  io_single[0].iTimeOut := ME_VALUE_NOT_USED;
  io_single[0].iFlags := flags;

  meError := meIOSingle(io_single[0],
                        1,
                        ME_IO_SINGLE_NO_FLAGS);

  value := io_single[0].iValue;
  Result := meError;
end;

procedure TfrmMain.DisplayReadBackValues(foSubdevice: TFoSubdevice;
  idxChannel: Integer; value1stPhase: Integer; valueTotal: Integer);
begin
  inc(lineNumber);
  memoInfo.Lines.Add(Format('%.2d', [lineNumber])
                     + ' Read from ' + foSubdevice.DeviceName
                     + ',  Device: ' + IntToStr(foSubdevice.IndexDevice)
                     + ',  Subdevice: ' + IntToStr(foSubdevice.IndexSubdevice)
                     + ',  Channel: ' + IntToStr(idxChannel));
  memoInfo.Lines.Add('  Ticks total: ' + IntToStr(valueTotal));
  memoInfo.Lines.Add('  Ticks first phase: ' + IntToStr(value1stPhase));
end;

function TfrmMain.ConfigureFoSubdevice(foSubdevice: TFoSubdevice; idxChannel: Integer): Integer;
var
  singleConfig: Integer;
  triggerChannel: Integer;
begin
  triggerChannel := GetOutputParam;
  singleConfig := GetSingleConfigParam(foSubdevice);
  Result := meIOSingleConfig(foSubdevice.IndexDevice,
                             foSubdevice.IndexSubdevice,
                             idxChannel,
                             singleConfig,
                             ME_VALUE_NOT_USED,
                             triggerChannel,
                             ME_TRIG_TYPE_NONE,
                             ME_VALUE_NOT_USED,
                             ME_IO_SINGLE_CONFIG_NO_FLAGS);
end;

procedure TfrmMain.btnDeviceWriteClick(Sender: TObject);
var
  meError: Integer;
  flags: Integer;
  idxChannel: Integer;
  ticksTotal: Integer;
  ticksFirstPhase: Integer;
  secondsAchievedTotal, secondsAchievedFirstPhase: Double;
  foSubdevice: TFoSubdevice;
begin
  if (foSubdevices.Count > 0) and (coBoxSubDevs.ItemIndex >= 0) then
  begin
    foSubdevice := foSubdevices.Items[coBoxSubDevs.ItemIndex];
    idxChannel := coBoxChannels.ItemIndex;

    memoInfo.Lines.Add(#13#10'Write Data');
    try
      secondsAchievedTotal := StrToFloat(editTotalTime.Text);
      secondsAchievedFirstPhase := StrToFloat(editFirstPhase.Text);

      meError := ConvertTimeToTicks(foSubdevice, ME_TIMER_FIO_TOTAL, secondsAchievedTotal, ticksTotal);
      if meError = ME_ERRNO_SUCCESS then
      begin

        flags := GetFoTotalTimeFlags();
        meError := WriteSingleValue(foSubdevice, idxChannel, flags, ticksTotal);
        if meError = ME_ERRNO_SUCCESS then
        begin

          meError := ConvertTimeToTicks(foSubdevice, ME_TIMER_FIO_FIRST_PHASE, secondsAchievedFirstPhase, ticksFirstPhase);
          if meError = ME_ERRNO_SUCCESS then
          begin

            flags := GetFoFirstPhaseTimeFlags();
            meError := WriteSingleValue(foSubdevice, idxChannel, flags, ticksFirstPhase);
            if meError = ME_ERRNO_SUCCESS then
            begin
              DisplayTimeValues(foSubdevice, idxChannel, secondsAchievedTotal, secondsAchievedFirstPhase, ticksTotal, ticksFirstPhase);
            end;
          end;
        end;
      end;
      if meError <> ME_ERRNO_SUCCESS then
      begin
        DisplayErrorMessage('Error: ', meError);
      end;
    except
      on e: Exception do
      begin
        memoInfo.Lines.Add('Error: ' + e.Message);
        MessageDlg('Error: ' + e.Message, mtError, [mbOK], 0);
      end;
    end;
  end;
end;

function TfrmMain.GetFoTotalTimeFlags(): Integer;
var
  iFlags: Integer;
begin
  iFlags := ME_IO_SINGLE_TYPE_NO_FLAGS;
  if rgWriteMode.ItemIndex = 1 then
  begin
    iFlags := iFlags or ME_IO_SINGLE_TYPE_TRIG_SYNCHRONOUS;
  end;
  iFlags := iFlags or ME_IO_SINGLE_TYPE_FIO_TICKS_TOTAL or ME_IO_SINGLE_TYPE_FO_UPDATE_ONLY;
  Result := iFlags;
end;

function TfrmMain.GetFoFirstPhaseTimeFlags(): Integer;
var
  iFlags: Integer;
begin
  iFlags := ME_IO_SINGLE_TYPE_NO_FLAGS;
  if rgWriteMode.ItemIndex = 1 then
  begin
    iFlags := iFlags or ME_IO_SINGLE_TYPE_TRIG_SYNCHRONOUS;
  end;
  if rgStartLevel.ItemIndex = 1 then
  begin
    iFlags := iFlags or ME_IO_SINGLE_TYPE_FO_START_LOW;
  end;
  if rgStartType.ItemIndex = 1 then
  begin
    iFlags := iFlags or ME_IO_SINGLE_TYPE_FO_START_SOFT;
  end;
  iFlags := iFlags or ME_IO_SINGLE_TYPE_FIO_TICKS_FIRST_PHASE;
  Result := iFlags;
end;

function TfrmMain.ConvertTimeToTicks(foSubdevice: TFoSubdevice; timer: Integer;
  var timeInSeconds: Double; var ticks: Integer): Integer;
var
  meError: Integer;
  ticksHigh: Integer;
begin
  meError := meIOSingleTimeToTicks(foSubdevice.IndexDevice,
                                   foSubdevice.IndexSubdevice,
                                   timer,
                                   timeInSeconds,
                                   ticks,
                                   ticksHigh,
                                   ME_IO_SINGLE_TIME_TO_TICKS_NO_FLAGS);

  Result := meError;
end;

procedure TfrmMain.DisplayTimeValues(foSubdevice: TFoSubdevice; idxChannel: Integer;
  secondsAchievedTotal, secondsAchievedFirstPhase: Double;
  ticksTotal, ticksFirstPhase: Integer);
const
  FORMAT_STR = '0.##########';
begin
  inc(lineNumber);
  memoInfo.Lines.Add(Format('%.2d', [lineNumber])
                     + ' Write to ' + foSubdevice.DeviceName
                     + ',  Device: ' + IntToStr(foSubdevice.IndexDevice)
                     + ',  Subdevice: ' + IntToStr(foSubdevice.IndexSubdevice)
                     + ',  Channel: ' + IntToStr(idxChannel));

  memoInfo.Lines.Add('  Total time: ' + FormatFloat(FORMAT_STR, secondsAchievedTotal)
                     + ' sec, frequency: ' + FormatFloat(FORMAT_STR, 1 / secondsAchievedTotal)
                     + ' Hz,  Ticks: ' + IntToStr(ticksTotal)
                    );

  memoInfo.Lines.Add('  First phase time: ' + FormatFloat(FORMAT_STR, secondsAchievedFirstPhase)
                     + ' sec,  Ticks: ' + IntToStr(ticksFirstPhase)
                    );

end;

function TfrmMain.WriteSingleValue(foSubdevice: TFoSubdevice; idxChannel,
  flags: Integer; value: Integer): Integer;
var
  io_single: array[0..0] of TMeIOSingle;
begin
  io_single[0].iDevice := foSubdevice.IndexDevice;
  io_single[0].iSubdevice := foSubdevice.IndexSubdevice;
  io_single[0].iChannel := idxChannel;
  io_single[0].iDir := ME_DIR_OUTPUT;
  io_single[0].iValue := value;
  io_single[0].iTimeOut := 0;
  io_single[0].iFlags := flags;

  Result := meIOSingle(io_single[0],
                        1,
                        ME_IO_SINGLE_NO_FLAGS);
end;


procedure TfrmMain.btnResetClick(Sender: TObject);
var
  meError: Integer;
  idxDevice: Integer;
  idxSubdevice: Integer;
  foSubdevice: TFoSubdevice;
begin
  if coBoxSubDevs.ItemIndex >= 0 then
  begin
    foSubdevice := foSubdevices.Items[coBoxSubDevs.ItemIndex];
    idxDevice := foSubdevice.IndexDevice;
    idxSubdevice := foSubdevice.IndexSubdevice;

    meError := meIOResetSubdevice(idxDevice,
                                  idxSubdevice,
                                  ME_IO_RESET_SUBDEVICE_NO_FLAGS);
    if meError = ME_ERRNO_SUCCESS then
    begin
      memoInfo.Lines.Add('Reset Device ' + IntToStr(idxDevice) + ', Subdevice ' + IntToStr(idxSubdevice));
    end else
    begin
      DisplayErrorMessage('meIOResetSubdevice - Error: ', meError);
    end;
  end;
end;

procedure TfrmMain.CloseMEiDSDriver();
begin
  if openErrSuccess = ME_ERRNO_SUCCESS then
  begin
    meClose(ME_CLOSE_NO_FLAGS);
    openErrSuccess := ME_IDS_NOTOPENED;
    coBoxSubDevs.Clear;
    coBoxChannels.Clear;
    foSubdevices.Clear;
    btnReadBack.Enabled := False;
    btnDeviceWrite.Enabled := False;
    btnReset.Enabled := False;
    btnSetConfig.Enabled := False;
    coBoxPortConfiguration.Enabled := False;
  end;
end;

procedure TfrmMain.coBoxSubDevsChange(Sender: TObject);
var
  foSubdevice: TFoSubdevice;
begin
  if coBoxSubDevs.ItemIndex >= 0 then
  begin
    foSubdevice := foSubdevices.Items[coBoxSubDevs.itemIndex];

    coBoxPortConfiguration.Enabled := (foSubdevice.SubdeviceCaps and ME_CAPS_FIO_SINK_SOURCE) = ME_CAPS_FIO_SINK_SOURCE;
    FillChannels(foSubdevice);
  end;
end;

procedure TfrmMain.FillChannels(foSubdevice: TFoSubdevice);
var
  i: Integer;
begin
  if coBoxChannels.Items.Count <> foSubdevice.NumberOfChannels then
  begin
    coBoxChannels.Clear;
    for i := 0 to foSubdevice.NumberOfChannels - 1 do
    begin
      coBoxChannels.Items.Add(IntToStr(i));
    end;
  end;
  if coBoxChannels.Items.Count > 0 then
    coBoxChannels.ItemIndex := 0;
end;

procedure TfrmMain.btnClearHwInfoClick(Sender: TObject);
begin
  memoInfo.Clear;
  lineNumber := 0;
end;

procedure TfrmMain.DisplayVersionLibrary;
var
  ver: Integer;
begin
  meQueryVersionLibrary(ver);
  memoInfo.Lines.Append('Library version: ' + MeVersionToString(ver));
end;

procedure TfrmMain.DisplayVersionMainDriver;
var
  ver: Integer;
begin
  meQueryVersionMainDriver(ver);
  memoInfo.Lines.Append('Main driver version: ' + MeVersionToString(ver));
end;

function TfrmMain.GetDevicePlugged(idxDevice: Integer; var plugged: Integer): Integer;
var
  busNo: Integer;
  vendorId: Integer;
  deviceId: Integer;
  funcNo: Integer;
  busType: Integer;
  serialNo: Integer;
  devNo: Integer;
begin
  Result := meQueryInfoDevice(idxDevice,
                    vendorId,
                    deviceId,
                    serialNo,
                    busType,
                    busNo,
                    devNo,
                    funcNo,
                    plugged);
end;

procedure TfrmMain.DisplayErrorMessage(ErrMsg: string; meError: Integer);
var
  errorMsg: array[0..ME_ERROR_MSG_MAX_COUNT] of AnsiChar;
begin
  meErrorGetMessage(meError, errorMsg, ME_ERROR_MSG_MAX_COUNT);
  memoInfo.lines.add(ErrMsg + IntToStr(meError));
  memoInfo.Lines.Add('ErrorMessage: ' + errorMsg);
end;

procedure TfrmMain.FillPortConfiguration;
begin
  coBoxPortConfiguration.AddItem('High impedance', TObject(ME_SINGLE_CONFIG_FIO_HIGH_IMPEDANCE));
  coBoxPortConfiguration.AddItem('Sink', TObject(ME_SINGLE_CONFIG_FIO_SINK));
  coBoxPortConfiguration.AddItem('Source', TObject(ME_SINGLE_CONFIG_FIO_SOURCE));
  coBoxPortConfiguration.ItemIndex := 0;
end;

function TfrmMain.MeVersionToString(version: Integer): string;
begin
  Result := Format('%d.%d.%d.%d', [version shr 24, $FF and (version shr 16), $FF and (version shr 8), $FF and version]);
end;

function TfrmMain.GetSubDevTypeName(subdeviceType: Integer): string;
begin
  case subdeviceType of
    ME_TYPE_INVALID: Result := 'ME_TYPE_INVALID';
    ME_TYPE_AI: Result := 'ME_TYPE_AI';
    ME_TYPE_AO: Result := 'ME_TYPE_AO';
    ME_TYPE_DIO: Result := 'ME_TYPE_DIO';
    ME_TYPE_DO: Result := 'ME_TYPE_DO';
    ME_TYPE_DI: Result := 'ME_TYPE_DI';
    ME_TYPE_CTR: Result := 'ME_TYPE_CTR';
    ME_TYPE_EXT_IRQ: Result := 'ME_TYPE_EXT_IRQ';
    ME_TYPE_FIO: Result := 'ME_TYPE_FIO';
    ME_TYPE_FO: Result := 'ME_TYPE_FO';
    ME_TYPE_FI: Result := 'ME_TYPE_FI';
    ME_TYPE_FPGA: Result := 'ME_TYPE_FPGA';
  else
    Result := '$' + IntToHex(subdeviceType, 8);
  end;
end;

procedure TfrmMain.btnAboutClick(Sender: TObject);
begin
  frmAbout := TfrmAbout.Create(Self);
  try
    frmAbout.PopupParent := Self;
    frmAbout.ShowModal;
  finally
    frmAbout.Free;
  end;
end;

end.
