﻿'==============================================================================
'Copyright(c), Meilhaus Electronic GmbH
'              http://www.meilhaus.de
'              support@meilhaus.de
'
'Source File : uMain.pas
'Description : Example program for using subdevices with Counter 8254
'              - Open main driver (ME-iDS)
'              - Go through the list of all installed ME-iDS devices
'                and find subdevices of type Counter and subtype 8254
'              - 3 counters at once can be selected from the subdevice combobox
'                as the integrated circuit 82C54 always contains 3 counters
'              - The counters can be used independently
'              - On some boards (e.g. ME-1400) counters of the same component
'                can be cascaded without external connections.
'              - Button "Write preload": Writes the preload value to the counter subdevice
'                and stops the timer if enabled
'              - "Single Read": single read from selected subdevice (counter)
'              - "Write & Run": writes the preload value to the subdevice
'                and starts a timer that reads the counter value periodically
'              - "Stop": stops the timer
'              - "Reset": stop the timer and reset the subdevice
'              - Close main driver when program is closed
'==============================================================================

Imports System.Text
Imports System.Runtime.InteropServices

Public Class frmMain
    Private openErrSuccess As Integer = Constants.ME_IDS_NOTOPENED
    Private lineNumber As Integer = 0
    Private subdeviceList As New List(Of CounterSubdevice)

    Private Sub frmMain_Shown(sender As System.Object, e As System.EventArgs) Handles MyBase.Shown
        coBoxMode1.SelectedIndex = 0
        coBoxMode2.SelectedIndex = 0
        coBoxMode3.SelectedIndex = 0

        OpenMEiDSDriver()
    End Sub

    Private Sub OpenMEiDSDriver()
        Try
            textBoxInfo.AppendText("Opening Maindriver..." & Environment.NewLine)
            openErrSuccess = meIDS.meOpen(meIDS.ME_OPEN_NO_FLAGS)

            If openErrSuccess = meIDS.ME_ERRNO_SUCCESS Then
                DisplayVersionLibrary()
                DisplayVersionMainDriver()

                textBoxInfo.AppendText(Environment.NewLine)

                Dim numDevices As Integer
                meIDS.meQueryNumberDevices(numDevices)
                textBoxInfo.AppendText("Number of devices found: " & numDevices.ToString() & Environment.NewLine)

                GetCounter8254Subdevices(numDevices)
            Else
                DisplayErrorMessage("meOpen - Error: ", openErrSuccess)
            End If
        Catch ex As DllNotFoundException
            textBoxInfo.AppendText("The ME-iDS is not properly installed." & Environment.NewLine)
            textBoxInfo.AppendText(ex.Message & Environment.NewLine)
        End Try
    End Sub

    Private Sub GetCounter8254Subdevices(numDevices As Integer)
        Dim idxDevice As Integer

        If openErrSuccess = meIDS.ME_ERRNO_SUCCESS AndAlso numDevices > 0 Then
            textBoxInfo.AppendText("Retrieving sub-devices of type ME_TYPE_CTR (Counter), subtype ME_SUBTYPE_CTR_8254 (8254)..." & Environment.NewLine)
            For idxDevice = 0 To numDevices - 1

                Dim meError As Integer
                Dim plugged As Integer
                meError = GetDevicePlugged(idxDevice, plugged)
                If meError = meIDS.ME_ERRNO_SUCCESS AndAlso plugged = meIDS.ME_PLUGGED_IN Then
                    GetSubdevicesByType(idxDevice, meIDS.ME_TYPE_CTR, meIDS.ME_SUBTYPE_CTR_8254)
                Else
                    textBoxInfo.AppendText("Device " & idxDevice.ToString() & " not plugged in" & Environment.NewLine)
                End If
            Next
        End If
        DisplaySubdevices()
    End Sub

    Private Sub GetSubdevicesByType(idxDevice As Integer, subdeviceType As Integer, subdeviceSubtype As Integer)
        Dim meError As Integer
        Dim numSubdevices As Integer

        meError = meIDS.meQueryNumberSubdevices(idxDevice, numSubdevices)
        If meError = meIDS.ME_ERRNO_SUCCESS Then
            Dim idxMatchedSubdevice As Integer
            Dim numMatched As Integer = 0
            Dim idxSubdevice As Integer = 0
            Dim deviceName = New StringBuilder(meIDS.ME_DEVICE_NAME_MAX_COUNT)

            meIDS.meQueryNameDevice(idxDevice, deviceName, deviceName.Capacity)
            While idxSubdevice < numSubdevices
                meError = meIDS.meQuerySubdeviceByType(idxDevice, _
                                                       idxSubdevice, _
                                                       subdeviceType, _
                                                       meIDS.ME_SUBTYPE_ANY, _
                                                       idxMatchedSubdevice)
                If meError = meIDS.ME_ERRNO_SUCCESS Then
                    numMatched = numMatched + 1
                    AddCountersText(deviceName.ToString(), idxDevice, idxMatchedSubdevice, numMatched)
                    AddCounterSubdevice(deviceName.ToString(), idxDevice, idxMatchedSubdevice)
                    idxSubdevice = idxMatchedSubdevice + 1
                Else
                    Exit While
                End If
            End While
        End If
    End Sub

    Private Sub AddCounterSubdevice(deviceName As String, idxDevice As Integer, idxSubdevice As Integer)
        Dim subdevCaps As Integer
        Dim args As Integer = 0
        meIDS.meQuerySubdeviceCaps(idxDevice, idxSubdevice, subdevCaps)
        meIDS.meQuerySubdeviceCapsArgs(idxDevice, idxSubdevice, meIDS.ME_CAP_CTR_WIDTH, args, 1)

        Dim ctrSubdevice As CounterSubdevice = New CounterSubdevice(deviceName.ToString(), idxDevice, idxSubdevice, subdevCaps, (1 << args) - 1)
        subdeviceList.Add(ctrSubdevice)
    End Sub

    Private Sub AddCountersText(deviceName As String, idxDevice As Integer, idxSubdevice As Integer, numMatched As Integer)
        Dim countersTxt As String
        If ((numMatched - 1) Mod 3) = 2 Then
            countersTxt = deviceName.ToString() _
                            & ", Device " & idxDevice.ToString() _
                            & String.Format(", Subdevices {0}, {1}, {2}", idxSubdevice - 2, idxSubdevice - 1, idxSubdevice)
            coBoxSubDevs.Items.Add(countersTxt)
        End If
    End Sub

    Private Sub ChangeCounterSubdevices(idx As Integer)
        If idx >= 0 Then
            StopAllTimers()

            Dim ctrSubdevice As CounterSubdevice = subdeviceList(idx * 3)
            coBoxMode1.SelectedIndex = 0
            FillCounterSources(ctrSubdevice, coBoxClockSource1)
            coBoxPreloadValue1.Text = ctrSubdevice.counterMaxValue.ToString()
            editCtrValue1.Text = "0"
            grpBoxCounter1.Text = String.Format("Subdevice {0}", ctrSubdevice.subdevIndex)

            coBoxMode2.SelectedIndex = 0
            ctrSubdevice = subdeviceList(idx * 3 + 1)
            FillCounterSources(ctrSubdevice, coBoxClockSource2)
            coBoxPreloadValue2.Text = ctrSubdevice.counterMaxValue.ToString()
            editCtrValue2.Text = "0"
            grpBoxCounter2.Text = String.Format("Subdevice {0}", ctrSubdevice.subdevIndex)

            coBoxMode3.SelectedIndex = 0
            ctrSubdevice = subdeviceList(idx * 3 + 2)
            FillCounterSources(ctrSubdevice, coBoxClockSource3)
            coBoxPreloadValue3.Text = ctrSubdevice.counterMaxValue.ToString()
            editCtrValue3.Text = "0"
            grpBoxCounter3.Text = String.Format("Subdevice {0}", ctrSubdevice.subdevIndex)
        End If
    End Sub

    Private Sub FillCounterSources(ctrSubdevice As CounterSubdevice, coBox As ComboBox)
        coBox.Items.Clear()
        If ((ctrSubdevice.subdevCaps And meIDS.ME_CAPS_CTR_CLK_EXTERNAL) = meIDS.ME_CAPS_CTR_CLK_EXTERNAL) Then
            coBox.Items.Add(New ComboBoxItem("External", meIDS.ME_REF_CTR_EXTERNAL))
        End If

        If ((ctrSubdevice.subdevCaps And meIDS.ME_CAPS_CTR_CLK_PREVIOUS) = meIDS.ME_CAPS_CTR_CLK_PREVIOUS) Then
            coBox.Items.Add(New ComboBoxItem("Previous", meIDS.ME_REF_CTR_PREVIOUS))
        End If

        If ((ctrSubdevice.subdevCaps And meIDS.ME_CAPS_CTR_CLK_INTERNAL_1MHZ) = meIDS.ME_CAPS_CTR_CLK_INTERNAL_1MHZ) Then
            coBox.Items.Add(New ComboBoxItem("Internal 1 MHz", meIDS.ME_REF_CTR_INTERNAL_1MHZ))
        End If

        If ((ctrSubdevice.subdevCaps And meIDS.ME_CAPS_CTR_CLK_INTERNAL_10MHZ) = meIDS.ME_CAPS_CTR_CLK_INTERNAL_10MHZ) Then
            coBox.Items.Add(New ComboBoxItem("Internal 10 MHz", meIDS.ME_REF_CTR_INTERNAL_10MHZ))
        End If

        If (coBox.Items.Count > 0) Then
            coBox.SelectedIndex = 0
        End If
    End Sub

    Private Function SingleReadCounter(ctrIdx As Integer, editValue As TextBox) As Integer
        Dim meError As Integer = meIDS.ME_ERRNO_SUCCESS
        Dim value As Integer

        If (coBoxSubDevs.SelectedIndex >= 0) Then
            meError = ReadCounterValue(ctrIdx, value)
            If (meError = meIDS.ME_ERRNO_SUCCESS) Then
                editValue.Text = value.ToString()
            Else
                editValue.Text = "Error " & meError.ToString()
            End If
        End If
        Return meError
    End Function

    Private Function ReadCounterValue(ctrIdx As Integer, ByRef value As Integer)
        Dim ctrSubdevice As CounterSubdevice = subdeviceList(coBoxSubDevs.SelectedIndex * 3 + ctrIdx)
        Dim io_single(1) As meIDS.meIOSingle_t
        Dim meError As Integer

        io_single(0).iDevice = ctrSubdevice.deviceIndex
        io_single(0).iSubdevice = ctrSubdevice.subdevIndex
        io_single(0).iChannel = 0
        io_single(0).iDir = meIDS.ME_DIR_INPUT
        io_single(0).iValue = 0
        io_single(0).iTimeOut = 0
        io_single(0).iFlags = meIDS.ME_IO_SINGLE_TYPE_NO_FLAGS
        io_single(0).iErrno = 0

        meError = meIDS.meIOSingle(io_single(0), _
                                   1, _
                                   meIDS.ME_IO_SINGLE_NO_FLAGS)

        value = io_single(0).iValue
        Return meError
    End Function


    Private Function SingleWriteCounter(ctrIdx As Integer) As Boolean
        Dim result As Boolean = False

        If coBoxSubDevs.SelectedIndex >= 0 Then
            Dim ctrSubdevice As CounterSubdevice = subdeviceList(coBoxSubDevs.SelectedIndex * 3 + ctrIdx)
            Dim ctrMode As Integer
            Dim ctrModeStr As String
            Dim preloadVal As Integer
            Dim ctrClockSource As Integer
            Dim ctrClockSrcStr As String
            Dim meError As Integer

            Try
                Select Case ctrIdx
                    Case 0
                        preloadVal = Integer.Parse(coBoxPreloadValue1.Text)
                        AddCoBoxValue(coBoxPreloadValue1, coBoxPreloadValue1.Text)
                        ctrMode = Constants.Ctr8254Modes(coBoxMode1.SelectedIndex)
                        ctrModeStr = coBoxMode1.Text
                        ctrClockSource = coBoxClockSource1.SelectedItem.GetValue
                        ctrClockSrcStr = coBoxClockSource1.Text
                    Case 1
                        preloadVal = Integer.Parse(coBoxPreloadValue2.Text)
                        AddCoBoxValue(coBoxPreloadValue2, coBoxPreloadValue2.Text)
                        ctrMode = Constants.Ctr8254Modes(coBoxMode2.SelectedIndex)
                        ctrModeStr = coBoxMode2.Text
                        ctrClockSource = coBoxClockSource2.SelectedItem.GetValue
                        ctrClockSrcStr = coBoxClockSource2.Text
                    Case 2
                        preloadVal = Integer.Parse(coBoxPreloadValue3.Text)
                        AddCoBoxValue(coBoxPreloadValue3, coBoxPreloadValue3.Text)
                        ctrMode = Constants.Ctr8254Modes(coBoxMode3.SelectedIndex)
                        ctrModeStr = coBoxMode3.Text
                        ctrClockSource = coBoxClockSource3.SelectedItem.GetValue
                        ctrClockSrcStr = coBoxClockSource3.Text
                    Case Else
                        Return False
                End Select

                If (preloadVal < 0 Or preloadVal > ctrSubdevice.counterMaxValue) Then
                    Throw New System.ArgumentOutOfRangeException("Preload value is out of range. Allowed values are from 0..." & ctrSubdevice.counterMaxValue.ToString())
                End If

                meError = ConfigureCounterSubdevice(ctrSubdevice, ctrMode, ctrClockSource)
                If (meError <> meIDS.ME_ERRNO_SUCCESS) Then
                    DisplayErrorMessage("meIOSingleConfig - Error: ", meError)
                    Return False
                End If

                meError = WritePreloadValue(ctrSubdevice, preloadVal)
                If (meError = meIDS.ME_ERRNO_SUCCESS) Then
                    DisplayValues(ctrSubdevice, ctrModeStr, ctrClockSrcStr, preloadVal)
                    result = True
                Else
                    DisplayErrorMessage("meIOSingle - Error: ", meError)
                End If
            Catch argEx As System.ArgumentOutOfRangeException
                MessageBox.Show(argEx.Message, MyBase.Text, MessageBoxButtons.OK, MessageBoxIcon.Error)
            Catch formatEx As System.FormatException
                MessageBox.Show(formatEx.Message, MyBase.Text, MessageBoxButtons.OK, MessageBoxIcon.Error)
            End Try
        End If
        Return result
    End Function

    Private Sub DisplayValues(ctrSubdevice As CounterSubdevice, ctrModeStr As String, ctrClockSrcStr As String, preloadVal As Integer)
        lineNumber = lineNumber + 1
        textBoxInfo.AppendText(String.Format("{0:00}", lineNumber) _
               & " Write to " + ctrSubdevice.deviceName _
               & ",  Device: " + ctrSubdevice.deviceIndex.ToString() _
               & ",  Subdevice: " + ctrSubdevice.subdevIndex.ToString() _
               & ",  Counter mode: " + ctrModeStr _
               & ",  Clock source: " + ctrClockSrcStr _
               & ",  Preload value: " + preloadVal.ToString() _
               & Environment.NewLine)
    End Sub

    Private Function ConfigureCounterSubdevice(ctrSubdevice As CounterSubdevice, ctrMode As Integer, ctrClockSource As Integer)
        Dim meError As Integer = meIDS.meIOSingleConfig(ctrSubdevice.deviceIndex, _
                                        ctrSubdevice.subdevIndex, _
                                        0, _
                                        ctrMode, _
                                        ctrClockSource, _
                                        meIDS.ME_TRIG_CHAN_DEFAULT, _
                                        meIDS.ME_TRIG_TYPE_NONE, _
                                        meIDS.ME_VALUE_NOT_USED, _
                                        meIDS.ME_IO_SINGLE_CONFIG_NO_FLAGS)
        Return meError
    End Function

    Private Function WritePreloadValue(ctrSubdevice As CounterSubdevice, preloadVal As Integer) As Integer
        Dim io_single(1) As meIDS.meIOSingle_t
        io_single(0).iDevice = ctrSubdevice.deviceIndex
        io_single(0).iSubdevice = ctrSubdevice.subdevIndex
        io_single(0).iChannel = 0
        io_single(0).iDir = meIDS.ME_DIR_OUTPUT
        io_single(0).iValue = preloadVal
        io_single(0).iTimeOut = 0
        io_single(0).iFlags = meIDS.ME_IO_SINGLE_TYPE_NO_FLAGS
        io_single(0).iErrno = 0

        Dim meError As Integer = meIDS.meIOSingle(io_single(0), _
                                        1, _
                                        meIDS.ME_IO_SINGLE_NO_FLAGS)
        Return meError
    End Function

    Private Sub ResetSubdevice(ctrIdx As Integer)
        If (coBoxSubDevs.SelectedIndex >= 0) Then
            StopTimer(ctrIdx)
            Dim ctrSubdevice As CounterSubdevice = subdeviceList(coBoxSubDevs.SelectedIndex * 3 + ctrIdx)

            Dim meError As Integer = meIDS.meIOResetSubdevice(ctrSubdevice.deviceIndex, _
                                          ctrSubdevice.subdevIndex, _
                                          meIDS.ME_IO_RESET_SUBDEVICE_NO_FLAGS)
            If (meError = meIDS.ME_ERRNO_SUCCESS) Then
                textBoxInfo.AppendText("Reset Device " & ctrSubdevice.deviceIndex.ToString() & ", Subdevice " & ctrSubdevice.subdevIndex.ToString() & Environment.NewLine)
            Else
                DisplayErrorMessage("meIOResetSubdevice - Error: ", meError)
            End If
        End If
    End Sub

    Private Sub btnStop1_Click(sender As System.Object, e As System.EventArgs) Handles btnStop1.Click
        StopTimer(0)
    End Sub

    Private Sub btnStop2_Click(sender As System.Object, e As System.EventArgs) Handles btnStop2.Click
        StopTimer(1)
    End Sub

    Private Sub btnStop3_Click(sender As System.Object, e As System.EventArgs) Handles btnStop3.Click
        StopTimer(2)
    End Sub

    Private Sub StopTimer(ctrIdx As Integer)
        Select Case ctrIdx
            Case 0
                StopTimer1()
            Case 1
                StopTimer2()
            Case 2
                StopTimer3()
            Case Else
                StopAllTimers()
        End Select
    End Sub

    Private Sub StopTimer1()
        tmr1.Enabled = False
        btnRun1.Enabled = True
    End Sub

    Private Sub StopTimer2()
        tmr2.Enabled = False
        btnRun2.Enabled = True
    End Sub

    Private Sub StopTimer3()
        tmr3.Enabled = False
        btnRun3.Enabled = True
    End Sub

    Private Sub StopAllTimers()
        StopTimer1()
        StopTimer2()
        StopTimer3()
    End Sub

    Private Sub frmMain_FormClosing(sender As System.Object, e As System.Windows.Forms.FormClosingEventArgs) Handles MyBase.FormClosing
        CloseMEiDSDriver()
    End Sub

    Private Sub btnClose_Click(sender As System.Object, e As System.EventArgs) Handles btnClose.Click
        Close()
    End Sub

    Private Sub btnClearHwInfo_Click(sender As System.Object, e As System.EventArgs) Handles btnClearHwInfo.Click
        textBoxInfo.Clear()
        lineNumber = 0
    End Sub

    Private Sub coBoxClockSource1_SelectedIndexChanged(sender As System.Object, e As System.EventArgs) Handles coBoxClockSource1.SelectedIndexChanged
        If tmr1.Enabled Then
            StopTimer1()
        End If
    End Sub

    Private Sub coBoxClockSource2_SelectedIndexChanged(sender As System.Object, e As System.EventArgs) Handles coBoxClockSource2.SelectedIndexChanged
        If tmr2.Enabled Then
            StopTimer2()
        End If
    End Sub

    Private Sub coBoxClockSource3_SelectedIndexChanged(sender As System.Object, e As System.EventArgs) Handles coBoxClockSource3.SelectedIndexChanged
        If tmr3.Enabled Then
            StopTimer3()
        End If
    End Sub

    Private Sub coBoxMode1_SelectedIndexChanged(sender As System.Object, e As System.EventArgs) Handles coBoxMode1.SelectedIndexChanged
        If tmr1.Enabled Then
            StopTimer1()
        End If
    End Sub

    Private Sub coBoxMode2_SelectedIndexChanged(sender As System.Object, e As System.EventArgs) Handles coBoxMode2.SelectedIndexChanged
        If tmr2.Enabled Then
            StopTimer2()
        End If
    End Sub

    Private Sub coBoxMode3_SelectedIndexChanged(sender As System.Object, e As System.EventArgs) Handles coBoxMode3.SelectedIndexChanged
        If tmr3.Enabled Then
            StopTimer3()
        End If
    End Sub

    Private Sub btnPreload1_Click(sender As System.Object, e As System.EventArgs) Handles btnPreload1.Click
        StopTimer1()
        SingleWriteCounter(0)
    End Sub

    Private Sub btnPreload2_Click(sender As System.Object, e As System.EventArgs) Handles btnPreload2.Click
        StopTimer2()
        SingleWriteCounter(1)
    End Sub

    Private Sub btnPreload3_Click(sender As System.Object, e As System.EventArgs) Handles btnPreload3.Click
        StopTimer3()
        SingleWriteCounter(2)
    End Sub

    Private Sub tmr1_Tick(sender As System.Object, e As System.EventArgs) Handles tmr1.Tick
        Dim meError As Integer = SingleReadCounter(0, editCtrValue1)
        If (meError <> meIDS.ME_ERRNO_SUCCESS AndAlso meError <> meIDS.ME_ERRNO_SUBDEVICE_NOT_RUNNING) Then
            StopTimer1()
            DisplayErrorMessage(grpBoxCounter1.Text & " meIOSingle Error: ", meError)
        End If
    End Sub

    Private Sub tmr2_Tick(sender As System.Object, e As System.EventArgs) Handles tmr2.Tick
        Dim meError As Integer = SingleReadCounter(1, editCtrValue2)
        If (meError <> meIDS.ME_ERRNO_SUCCESS AndAlso meError <> meIDS.ME_ERRNO_SUBDEVICE_NOT_RUNNING) Then
            StopTimer2()
            DisplayErrorMessage(grpBoxCounter2.Text & " meIOSingle Error: ", meError)
        End If
    End Sub

    Private Sub tmr3_Tick(sender As System.Object, e As System.EventArgs) Handles tmr3.Tick
        Dim meError As Integer = SingleReadCounter(2, editCtrValue3)
        If (meError <> meIDS.ME_ERRNO_SUCCESS AndAlso meError <> meIDS.ME_ERRNO_SUBDEVICE_NOT_RUNNING) Then
            StopTimer3()
            DisplayErrorMessage(grpBoxCounter3.Text & " meIOSingle Error: ", meError)
        End If
    End Sub

    Private Sub btnRun1_Click(sender As System.Object, e As System.EventArgs) Handles btnRun1.Click
        btnRun1.Enabled = False
        If (SingleWriteCounter(0)) Then
            tmr1.Enabled = True
        Else
            btnRun1.Enabled = True
        End If
    End Sub

    Private Sub btnRun2_Click(sender As System.Object, e As System.EventArgs) Handles btnRun2.Click
        btnRun2.Enabled = False
        If (SingleWriteCounter(1)) Then
            tmr2.Enabled = True
        Else
            btnRun2.Enabled = True
        End If
    End Sub

    Private Sub btnRun3_Click(sender As System.Object, e As System.EventArgs) Handles btnRun3.Click
        btnRun3.Enabled = False
        If (SingleWriteCounter(2)) Then
            tmr3.Enabled = True
        Else
            btnRun3.Enabled = True
        End If
    End Sub

    Private Sub btnRead1_Click(sender As System.Object, e As System.EventArgs) Handles btnRead1.Click
        Dim meError As Integer = SingleReadCounter(0, editCtrValue1)
        If (meError <> meIDS.ME_ERRNO_SUCCESS) Then
            DisplayErrorMessage(grpBoxCounter1.Text & " meIOSingle Error: ", meError)
        End If
    End Sub

    Private Sub btnRead2_Click(sender As System.Object, e As System.EventArgs) Handles btnRead2.Click
        Dim meError As Integer = SingleReadCounter(1, editCtrValue2)
        If (meError <> meIDS.ME_ERRNO_SUCCESS) Then
            DisplayErrorMessage(grpBoxCounter2.Text & " meIOSingle Error: ", meError)
        End If
    End Sub

    Private Sub btnRead3_Click(sender As System.Object, e As System.EventArgs) Handles btnRead3.Click
        Dim meError As Integer = SingleReadCounter(2, editCtrValue3)
        If (meError <> meIDS.ME_ERRNO_SUCCESS) Then
            DisplayErrorMessage(grpBoxCounter3.Text & " meIOSingle Error: ", meError)
        End If
    End Sub

    Private Sub btnReset1_Click(sender As System.Object, e As System.EventArgs) Handles btnReset1.Click
        ResetSubdevice(0)
        editCtrValue1.Text = "0"
    End Sub

    Private Sub btnReset2_Click(sender As System.Object, e As System.EventArgs) Handles btnReset2.Click
        ResetSubdevice(1)
        editCtrValue2.Text = "0"
    End Sub

    Private Sub btnReset3_Click(sender As System.Object, e As System.EventArgs) Handles btnReset3.Click
        ResetSubdevice(2)
        editCtrValue3.Text = "0"
    End Sub

    Private Sub DisplaySubdevices()
        If (coBoxSubDevs.Items.Count > 0) Then
            textBoxInfo.AppendText(String.Format("Number of Counter sub-devices found: {0}", coBoxSubDevs.Items.Count * 3) & Environment.NewLine)
            Dim i As Integer
            For i = 0 To coBoxSubDevs.Items.Count - 1
                textBoxInfo.AppendText(coBoxSubDevs.Items(i) & Environment.NewLine)
            Next
            textBoxInfo.AppendText(Environment.NewLine)
            coBoxSubDevs.SelectedIndex = 0
            ChangeCounterSubdevices(coBoxSubDevs.SelectedIndex)
        Else
            textBoxInfo.AppendText("No Counter sub-devices of subtype 8254 found." & Environment.NewLine)
            CloseMEiDSDriver()
        End If
    End Sub

    Private Sub DisplayVersionLibrary()
        Dim tmpVersion As Integer
        meIDS.meQueryVersionLibrary(tmpVersion)
        textBoxInfo.AppendText("Library version: " & MeVersionToString(tmpVersion) & Environment.NewLine)
    End Sub

    Private Sub DisplayVersionMainDriver()
        Dim tmpVersion As Integer
        meIDS.meQueryVersionMainDriver(tmpVersion)
        textBoxInfo.AppendText("Maindriver version: " & MeVersionToString(tmpVersion) & Environment.NewLine)
    End Sub

    Private Sub CloseMEiDSDriver()
        StopAllTimers()
        If (openErrSuccess = meIDS.ME_ERRNO_SUCCESS) Then
            meIDS.meClose(meIDS.ME_CLOSE_NO_FLAGS)
            openErrSuccess = Constants.ME_IDS_NOTOPENED
            subdeviceList.Clear()
            coBoxSubDevs.Items.Clear()
        End If
    End Sub

    Private Function GetDevicePlugged(idxDevice As Integer, ByRef plugged As Integer) As Integer

        Dim meError As Integer
        Dim vendorId As Integer
        Dim deviceId As Integer
        Dim serialNo As Integer
        Dim busType As Integer
        Dim busNo As Integer
        Dim devNo As Integer
        Dim funcNo As Integer

        meError = meIDS.meQueryInfoDevice(idxDevice, _
                                    vendorId, _
                                    deviceId, _
                                    serialNo, _
                                    busType, _
                                    busNo, _
                                    devNo, _
                                    funcNo, _
                                    plugged)
        Return meError
    End Function

    Private Function MeVersionToString(version As Integer) As String
        Return String.Format("{0}.{1}.{2}.{3}", version >> 24, &HFF And (version >> 16), &HFF And (version >> 8), &HFF And version)
    End Function

    Private Sub DisplayErrorMessage(message As String, meError As Integer)
        Dim errMessage = New StringBuilder(meIDS.ME_ERROR_MSG_MAX_COUNT)
        textBoxInfo.AppendText(message & meError.ToString() & Environment.NewLine)
        meIDS.meErrorGetMessage(meError, errMessage, meIDS.ME_ERROR_MSG_MAX_COUNT)
        textBoxInfo.AppendText("ErrorMessage: " & errMessage.ToString & Environment.NewLine)
    End Sub

    Private Sub AddCoBoxValue(coBox As ComboBox, newStr As String)
        Dim tmpStr As String = newStr.Trim()
        If tmpStr.Length > 0 AndAlso coBox.Items.IndexOf(tmpStr) < 0 Then
            coBox.Items.Add(tmpStr)
        End If
    End Sub

    Private Sub btnAbout_Click(sender As System.Object, e As System.EventArgs) Handles btnAbout.Click
        AboutBox1.ShowDialog()
    End Sub
End Class

Public Class CounterSubdevice
    Public deviceName As String
    Public deviceIndex As Integer
    Public subdevIndex As Integer
    Public subdevCaps As Integer
    Public counterMaxValue As Integer
    Sub New(DevName As String, DeviceIdx As Integer, SubdevIdx As Integer, Capabilities As Integer, CtrMaxValue As Integer)
        deviceName = DevName
        deviceIndex = DeviceIdx
        subdevIndex = SubdevIdx
        subdevCaps = Capabilities
        counterMaxValue = CtrMaxValue
    End Sub

End Class

Public Class ComboBoxItem
    Dim dispValue As String
    Dim val As Integer

    Sub New(displayValue As String, value As Integer)
        dispValue = displayValue
        val = value
    End Sub

    Public Overrides Function ToString() As String
        Return dispValue
    End Function

    ReadOnly Property GetValue() As Integer
        Get
            Return val
        End Get
    End Property
End Class

Class Constants
    Public Shared ReadOnly ME_IDS_NOTOPENED As Integer = -1
    Public Shared ReadOnly Ctr8254Modes As Integer() = {meIDS.ME_SINGLE_CONFIG_CTR_8254_MODE_0,
                                                        meIDS.ME_SINGLE_CONFIG_CTR_8254_MODE_1,
                                                        meIDS.ME_SINGLE_CONFIG_CTR_8254_MODE_2,
                                                        meIDS.ME_SINGLE_CONFIG_CTR_8254_MODE_3,
                                                        meIDS.ME_SINGLE_CONFIG_CTR_8254_MODE_4,
                                                        meIDS.ME_SINGLE_CONFIG_CTR_8254_MODE_5}


End Class

