﻿'==============================================================================
'Copyright(c), Meilhaus Electronic GmbH
'              http://www.meilhaus.de
'              support@meilhaus.de
'
'Source File : FormMain.vb
'Description : Example program for reading values from Frequency-In (FI) subdevices
'
'==============================================================================
'  Note about the hardware:
'  After meIOSingle is called (start of the measurement operation) 
'  the FI hardware will automatically detect the next rising edge 
'  of the unknown digital signal to start the measurement of the signal period.
'  The first counter (counter A) on the hardware is started.
'  Counter A is stopped when the next falling edge is detected.
'  At the same time a second counter (counter B) is started that will stop on
'  detection of the next rising edge (the end of the signal period).
'  The two counter values describe the signal period
'  The ME-iDS function library will provide two values
'  1. the counter value for the entire signal period (counter A + counter B)
'  2. the counter value of the 1st phase of the signal period (counter A)
'
'  Reading Data:
'  A new measurement operation is started with the given timeout value
'  (iFlag is set to ME_IO_SINGLE_CONFIG_NO_FLAGS)
'  After the measuring is done (meIOSingle() returned successfully)
'  the value of Total ticks is returned in member iValue of meIOSingle_t,
'  Then meIOSingle is called a second time to read the value of the first phase.
'  Therefore ME_IO_SINGLE_TYPE_FI_LAST_VALUE must be specified in iFlags
'  (If ME_IO_SINGLE_TYPE_FI_LAST_VALUE isn't specified a new measuring
'  would be started and the returned counter value wouldn't belong
'  to the same signal period, thus giving wrong results)
'  With the function meIOSingleTicksToTime() these values can be converted to a time value.
'
'  unknown digital signal:
'      +-------+                   +---
'      |       |                   |   
'      |       |                   |   
'  ----+       +-------------------+   
'                                   
'  ^---^-------^-------------------^
'  0   1       2                   3
'
'  0: Start of the operation (call of function meIOSingle())
'  1: Start of hardware counter A on rising edge of the signal
'  2: End of the first phase of the signal, stop of hardware counter A
'     Start of hardware counter B
'  3: End of the second phase of the signal, stop of hardware counter B
'     End of the operation (function meIOSingle() returned)
'
'
'------------------------------------------------------------------------------
Imports System.Text

Public Class frmMain
    Private openErrSuccess As Integer = Constants.ME_IDS_NOTOPENED
    Private lineNumber As Integer
    Private subdeviceList As New List(Of FiSubdevice)

    Private Sub frmMain_Shown(sender As System.Object, e As System.EventArgs) Handles MyBase.Shown
        OpenMEiDSDriver()
    End Sub

    Private Sub frmMain_FormClosing(sender As System.Object, e As System.Windows.Forms.FormClosingEventArgs) Handles MyBase.FormClosing
        CloseMEiDSDriver()
    End Sub

    Private Sub btnClose_Click(sender As System.Object, e As System.EventArgs) Handles btnClose.Click
        Close()
    End Sub

    Private Sub OpenMEiDSDriver()
        Try
            textBoxInfo.AppendText("Opening Maindriver..." & Environment.NewLine)
            openErrSuccess = meIDS.meOpen(meIDS.ME_OPEN_NO_FLAGS)

            If openErrSuccess = meIDS.ME_ERRNO_SUCCESS Then
                DisplayVersionLibrary()
                DisplayVersionMainDriver()

                textBoxInfo.AppendText(Environment.NewLine)

                Dim numDevices As Integer
                meIDS.meQueryNumberDevices(numDevices)
                textBoxInfo.AppendText("Number of devices found: " & numDevices.ToString() & Environment.NewLine)
                GetFiSubdevices(numDevices)
            Else
                DisplayErrorMessage("meOpen - Error: ", openErrSuccess)
            End If
        Catch ex As DllNotFoundException
            textBoxInfo.AppendText("The ME-iDS is not properly installed." & Environment.NewLine)
            textBoxInfo.AppendText(ex.Message & Environment.NewLine)
        End Try
    End Sub

    Private Sub GetFiSubdevices(numDevices As Integer)
        If openErrSuccess = meIDS.ME_ERRNO_SUCCESS AndAlso numDevices > 0 Then
            textBoxInfo.AppendText("Retrieving subdevices of type ME_TYPE_FI (Frequency In)..." & Environment.NewLine)
            For idxDevice = 0 To numDevices - 1
                Dim meError As Integer
                Dim plugged As Integer
                meError = GetDevicePlugged(idxDevice, plugged)
                If meError = meIDS.ME_ERRNO_SUCCESS AndAlso plugged = meIDS.ME_PLUGGED_IN Then
                    GetSubdevicesByType(idxDevice, meIDS.ME_TYPE_FI)
                Else
                    textBoxInfo.AppendText("Device " & idxDevice.ToString() & " not plugged in" & Environment.NewLine)
                End If
            Next
        End If
        DisplaySubdevices()
    End Sub

    Private Sub GetSubdevicesByType(idxDevice As Integer, subdeviceType As Integer)
        Dim meError As Integer
        Dim numSubdevices As Integer
        Dim idxMatchedSubdevice As Integer

        meError = meIDS.meQueryNumberSubdevices(idxDevice, numSubdevices)
        If meError = meIDS.ME_ERRNO_SUCCESS Then
            Dim devName = New StringBuilder(meIDS.ME_DEVICE_NAME_MAX_COUNT)
            meIDS.meQueryNameDevice(idxDevice, devName, devName.Capacity)
            Dim idxSubdevice As Integer = 0
            While idxSubdevice < numSubdevices
                meError = meIDS.meQuerySubdeviceByType(idxDevice, _
                                                       idxSubdevice, _
                                                       subdeviceType, _
                                                       meIDS.ME_SUBTYPE_ANY, _
                                                       idxMatchedSubdevice)

                If (meError = meIDS.ME_ERRNO_SUCCESS) Then
                    AddFiSubdevice(devName.ToString(), idxDevice, idxMatchedSubdevice, subdeviceType)
                    idxSubdevice = idxMatchedSubdevice + 1
                Else
                    Exit While
                End If
            End While
        End If
    End Sub

    Private Sub AddFiSubdevice(devName As String, idxDevice As Integer, idxSubdevice As Integer, subdeviceType As Integer)
        Dim numOfChannels As Integer
        meIDS.meQueryNumberChannels(idxDevice, idxSubdevice, numOfChannels)
        subdeviceList.Add(New FiSubdevice(devName, idxDevice, idxSubdevice, numOfChannels))
        coBoxSubDevs.Items.Add(devName _
                                & ",  Device " & idxDevice.ToString() _
                                & ",  Subdevice " & idxSubdevice.ToString() _
                                & ",  Type: " & GetSubDevTypeName(subdeviceType) _
                                & ",  Channels: " & numOfChannels.ToString())
    End Sub


    Private Sub btnResetSubdev_Click(sender As System.Object, e As System.EventArgs) Handles btnResetSubdev.Click
        Dim fiSubdev As FiSubdevice
        Dim meError As Integer
        fiSubdev = subdeviceList(coBoxSubDevs.SelectedIndex)
        meError = meIDS.meIOResetSubdevice(fiSubdev.deviceIndex, _
                                              fiSubdev.subdevIndex, _
                                              meIDS.ME_IO_RESET_SUBDEVICE_NO_FLAGS)
        If meError = meIDS.ME_ERRNO_SUCCESS Then
            textBoxInfo.AppendText("Reset Device " & fiSubdev.deviceIndex.ToString() & ", Subdevice " & fiSubdev.subdevIndex.ToString() & Environment.NewLine)
        Else
            displayErrorMessage("Error resetting Subdevice", meError)
        End If
    End Sub

    Private Sub coBoxSubDevs_SelectedIndexChanged(sender As System.Object, e As System.EventArgs) Handles coBoxSubDevs.SelectedIndexChanged
        coBoxChannel.Items.Clear()
        If (coBoxSubDevs.SelectedIndex >= 0) Then
            Dim fiSubdevice As FiSubdevice
            fiSubdevice = subdeviceList(coBoxSubDevs.SelectedIndex)
            Dim i As Integer
            For i = 0 To fiSubdevice.numOfChannels - 1
                coBoxChannel.Items.Add(i.ToString())
            Next
        End If
        If (coBoxChannel.Items.Count > 0) Then
            coBoxChannel.SelectedIndex = 0
        End If
    End Sub

    Private Sub btnSingleRead_Click(sender As System.Object, e As System.EventArgs) Handles btnSingleRead.Click
        If (coBoxSubDevs.SelectedIndex >= 0) Then
            EnableUI(False)
            Try
                Dim meError As Integer
                Dim fiSubdevice As FiSubdevice
                fiSubdevice = subdeviceList(coBoxSubDevs.SelectedIndex)
                Dim idxChannel As Integer = coBoxChannel.SelectedIndex
                Dim timeout As Integer = numUpDownTimeout.Value

                meError = ConfigureSubdeviceForInput(fiSubdevice, idxChannel)
                If (meError = meIDS.ME_ERRNO_SUCCESS) Then
                    Dim valueTotal As Integer
                    Dim flags As Integer = meIDS.ME_IO_SINGLE_TYPE_FIO_TICKS_TOTAL
                    meError = ReadSingleValue(fiSubdevice, idxChannel, timeout, flags, valueTotal)
                    If (meError = meIDS.ME_ERRNO_SUCCESS) Then
                        Dim value1stPhase As Integer
                        flags = meIDS.ME_IO_SINGLE_TYPE_FI_LAST_VALUE Or meIDS.ME_IO_SINGLE_TYPE_FIO_TICKS_FIRST_PHASE
                        meError = ReadSingleValue(fiSubdevice, idxChannel, timeout, flags, value1stPhase)
                        If (meError = meIDS.ME_ERRNO_SUCCESS) Then
                            meError = DisplayValues(fiSubdevice, idxChannel, valueTotal, value1stPhase)
                        End If
                    End If

                    If meError <> meIDS.ME_ERRNO_SUCCESS Then
                        DisplayErrorMessage("Error: ", meError)
                    End If
                End If
            Finally
                EnableUI(True)
            End Try
        End If
    End Sub

    Private Function ConfigureSubdeviceForInput(fiSubdevice As FiSubdevice, idxChannel As Integer) As Integer
        Dim meError As Integer = meIDS.meIOSingleConfig(fiSubdevice.deviceIndex,
                                    fiSubdevice.subdevIndex,
                                    idxChannel,
                                    meIDS.ME_SINGLE_CONFIG_FIO_INPUT,
                                    meIDS.ME_VALUE_NOT_USED,
                                    meIDS.ME_TRIG_CHAN_DEFAULT,
                                    meIDS.ME_TRIG_TYPE_NONE,
                                    meIDS.ME_VALUE_NOT_USED,
                                    meIDS.ME_IO_SINGLE_CONFIG_NO_FLAGS)
        Return meError
    End Function

    Private Function ReadSingleValue(fiSubdevice As FiSubdevice, idxChannel As Integer, timeout As Integer, flags As Integer, ByRef value As Integer) As Integer
        Dim io_single(1) As meIDS.meIOSingle_t

        io_single(0).iDevice = fiSubdevice.deviceIndex
        io_single(0).iSubdevice = fiSubdevice.subdevIndex
        io_single(0).iChannel = idxChannel
        io_single(0).iDir = meIDS.ME_DIR_INPUT
        io_single(0).iValue = 0
        io_single(0).iTimeOut = timeout
        io_single(0).iFlags = flags

        Dim meError As Integer = meIDS.meIOSingle(io_single(0), _
                                    1, _
                                    meIDS.ME_IO_SINGLE_NO_FLAGS)
        value = io_single(0).iValue
        Return meError
    End Function

    Private Function DisplayValues(fiSubdevice As FiSubdevice, idxChannel As Integer, valueTotal As Integer, value1stPhase As Integer) As Integer
        Dim firstPhaseTime As Double = 0
        Dim totalTime As Double = 0

        Dim meError As Integer = CalculateTime(fiSubdevice, value1stPhase, meIDS.ME_TIMER_FIO_FIRST_PHASE, firstPhaseTime)
        If (meError = meIDS.ME_ERRNO_SUCCESS) Then
            CalculateTime(fiSubdevice, valueTotal, meIDS.ME_TIMER_FIO_TOTAL, totalTime)

            lineNumber = lineNumber + 1
            textBoxInfo.AppendText(String.Format("{0:00}", lineNumber) _
                                    & " Read from " & fiSubdevice.deviceName _
                                    & ",  Device: " & fiSubdevice.deviceIndex.ToString() _
                                    & ",  Subdevice: " & fiSubdevice.subdevIndex.ToString() _
                                    & ",  Channel: " & idxChannel.ToString())
            textBoxInfo.AppendText("  Ticks total: " & valueTotal.ToString() _
                                    & String.Format(",  Seconds: {0:0.##########},  Hz: {1:0.##########}", totalTime, 1 / totalTime))
            textBoxInfo.AppendText(",  Ticks 1st phase: " & value1stPhase.ToString() _
                                    & String.Format(",  Seconds: {0:0.##########},  Duty Cycle: {1:0.###}%", firstPhaseTime, (valueTotal / value1stPhase) * 100))
        End If
        Return meError
    End Function

    Private Function CalculateTime(fiSubdevice As FiSubdevice, valueDigital As Integer, timer As Integer, ByRef time As Double)
        Dim meError As Integer = meIDS.meIOSingleTicksToTime(fiSubdevice.deviceIndex, _
                                        fiSubdevice.subdevIndex, _
                                        timer, _
                                        valueDigital, _
                                        0, _
                                        time, _
                                        meIDS.ME_IO_SINGLE_TICKS_TO_TIME_NO_FLAGS)

        Return meError
    End Function
    
    Private Sub CloseMEiDSDriver()
        If (openErrSuccess = meIDS.ME_ERRNO_SUCCESS) Then
            meIDS.meClose(meIDS.ME_CLOSE_NO_FLAGS)
            openErrSuccess = Constants.ME_IDS_NOTOPENED
            btnSingleRead.Enabled = False
            btnResetSubdev.Enabled = False
            subdeviceList.Clear()
            coBoxSubDevs.Items.Clear()
        End If
    End Sub

    Private Function GetSubDevTypeName(subdeviceType As Integer)
        Dim tmpType As String
        Select Case subdeviceType
            Case meIDS.ME_TYPE_INVALID
                tmpType = "ME_TYPE_INVALID"
            Case meIDS.ME_TYPE_AI
                tmpType = "ME_TYPE_AI"
            Case meIDS.ME_TYPE_AO
                tmpType = "ME_TYPE_AO"
            Case meIDS.ME_TYPE_DIO
                tmpType = "ME_TYPE_DIO"
            Case meIDS.ME_TYPE_DO
                tmpType = "ME_TYPE_DO"
            Case meIDS.ME_TYPE_DI
                tmpType = "ME_TYPE_DI"
            Case meIDS.ME_TYPE_CTR
                tmpType = "ME_TYPE_CTR"
            Case meIDS.ME_TYPE_EXT_IRQ
                tmpType = "ME_TYPE_EXT_IRQ"
            Case meIDS.ME_TYPE_FIO
                tmpType = "ME_TYPE_FIO"
            Case meIDS.ME_TYPE_FO
                tmpType = "ME_TYPE_FO"
            Case meIDS.ME_TYPE_FI
                tmpType = "ME_TYPE_FI"
            Case meIDS.ME_TYPE_FPGA
                tmpType = "ME_TYPE_FPGA"
            Case Else
                tmpType = String.Format("&H{0:X}", subdeviceType)
        End Select
        Return tmpType
    End Function

    Private Sub DisplayErrorMessage(message As String, meError As Integer)
        Dim errMessage = New StringBuilder(meIDS.ME_ERROR_MSG_MAX_COUNT)
        textBoxInfo.AppendText(message & meError.ToString() & Environment.NewLine)
        meIDS.meErrorGetMessage(meError, errMessage, meIDS.ME_ERROR_MSG_MAX_COUNT)
        textBoxInfo.AppendText("ErrorMessage: " & errMessage.ToString & Environment.NewLine)
    End Sub

    Private Sub EnableUI(isEnabled As Boolean)
        coBoxSubDevs.Enabled = isEnabled
        coBoxChannel.Enabled = isEnabled
        btnSingleRead.Enabled = isEnabled
        numUpDownTimeout.Enabled = isEnabled
    End Sub

    Private Sub DisplaySubdevices()
        If (coBoxSubDevs.Items.Count > 0) Then
            textBoxInfo.AppendText("Number of FI subdevices found: " & coBoxSubDevs.Items.Count.ToString() & Environment.NewLine)
            Dim i As Integer
            For i = 0 To coBoxSubDevs.Items.Count - 1
                textBoxInfo.AppendText(coBoxSubDevs.Items(i) & Environment.NewLine)
            Next
            textBoxInfo.AppendText(Environment.NewLine)
            coBoxSubDevs.SelectedIndex = 0
            btnResetSubdev.Enabled = True
            btnSingleRead.Enabled = True
        Else
            textBoxInfo.AppendText("No FI subdevices found in system." & Environment.NewLine)
            CloseMEiDSDriver()
        End If
    End Sub

    Private Sub DisplayVersionLibrary()
        Dim tmpVersion As Integer
        meIDS.meQueryVersionLibrary(tmpVersion)
        textBoxInfo.AppendText("Library version: " & MeVersionToString(tmpVersion) & Environment.NewLine)
    End Sub

    Private Sub DisplayVersionMainDriver()
        Dim tmpVersion As Integer
        meIDS.meQueryVersionMainDriver(tmpVersion)
        textBoxInfo.AppendText("Maindriver version: " & MeVersionToString(tmpVersion) & Environment.NewLine)
    End Sub

    Private Function GetDevicePlugged(idxDevice As Integer, ByRef plugged As Integer) As Integer

        Dim meError As Integer
        Dim vendorId As Integer
        Dim deviceId As Integer
        Dim serialNo As Integer
        Dim busType As Integer
        Dim busNo As Integer
        Dim devNo As Integer
        Dim funcNo As Integer

        meError = meIDS.meQueryInfoDevice(idxDevice, _
                                    vendorId, _
                                    deviceId, _
                                    serialNo, _
                                    busType, _
                                    busNo, _
                                    devNo, _
                                    funcNo, _
                                    plugged)
        Return meError
    End Function

    Private Function MeVersionToString(version As Integer) As String
        Return String.Format("{0}.{1}.{2}.{3}", version >> 24, &HFF And (version >> 16), &HFF And (version >> 8), &HFF And version)
    End Function

    Private Sub btnClearHwInfo_Click(sender As System.Object, e As System.EventArgs) Handles btnClearHwInfo.Click
        textBoxInfo.Clear()
        lineNumber = 0
    End Sub

    Private Sub btnAbout_Click(sender As System.Object, e As System.EventArgs) Handles btnAbout.Click
        AboutBox1.ShowDialog()
    End Sub
End Class

Public Class FiSubdevice
    Public deviceName As String
    Public deviceIndex As Integer
    Public subdevIndex As Integer
    Public numOfChannels As Integer
    Sub New(DevName As String, DeviceIdx As Integer, SubdevIdx As Integer, NumChannels As Integer)
        deviceName = DevName
        deviceIndex = DeviceIdx
        subdevIndex = SubdevIdx
        numOfChannels = NumChannels
    End Sub
End Class

Class Constants
    Public Shared ReadOnly ME_IDS_NOTOPENED As Integer = -1
End Class