﻿//==============================================================================
//Copyright(c) Meilhaus Electronic GmbH
//              http://www.meilhaus.de
//              support@meilhaus.de
//
//Source File : FormMain.cs
//Description : Example program for detecting bit changes (e.g. ME-8100, ME-8200, ME-5100)
//              - Open ME-iDS main driver when program is started
//              - Go through the list of all devices and find subdevices with Bit change capability
//              - Add matching subdevices to the combobox, so the user can select one
//              - Close ME-iDS main driver when program is closed
//==============================================================================
//Bitchange:
// This program can only work (detect changes) when an external signal is provided at the pins of the selected subdevice
// e.g. use of a bounce-free switch
// Two options are supported to get informed about a bitchange on a subdevice
// - Use meIOIrqSetCallback and a user defined callback function
//   Using a callback function allows the program to respond to other windows messages while waiting for a bitchange to occur
//   Note: The callback function is called from a different thread created by the ME-iDS library
// - Use meIOIrqWait means wait explicitly for an interrupt to arrive 
//   No extra thread is created by the ME-iDS library.

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Threading;

using meIDS_CS;

namespace BitChange
{
    public partial class frmMain : Form
    {
        private int openErrSuccess = Constants.ME_IDS_NOTOPENED;                
        private int dataLine = 0;                                               
        private List<BitChSubdevice> subdeviceList = new List<BitChSubdevice>(); 
        meIDS.meIOIrqCB_t IrqCallback = new meIDS.meIOIrqCB_t(meIOIrqCB);       
        GCHandle objectHandle = new GCHandle();
        CheckBox[] bitChBoxes = new CheckBox[32];
        Label[] bitIndicators = new Label[32];

        public frmMain()
        {
            InitializeComponent();
        }

        private void frmMain_Shown(object sender, EventArgs e)
        {
            InitBitMaskCheckBoxArray();
            InitBitIndicatorsArray();
            FillStatusFlags();

            OpenMEiDSDriver();
        }

        private void frmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            CloseMEiDSDriver();
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void OpenMEiDSDriver()
        {
            try
            {
                textBoxInfo.AppendText("Opening ME-iDS..." + Environment.NewLine);
                openErrSuccess = meIDS.meOpen(meIDS.ME_OPEN_NO_FLAGS);         
    
                if (openErrSuccess == meIDS.ME_ERRNO_SUCCESS)
                {
                    DisplayVersionLibrary();
                    DisplayVersionMainDriver();
    
                    int numDevices;
                    meIDS.meQueryNumberDevices(out numDevices);                     
                    textBoxInfo.AppendText("Number of devices found: " + numDevices.ToString() + Environment.NewLine);
    
                    GetSubdevicesWithBitchange(numDevices); 
                }
                else
                {
                    DisplayErrorMessage("Error opening ME-iDS. Error: ", openErrSuccess);
                }
            }
            catch (DllNotFoundException ex)
            {
                textBoxInfo.AppendText("The ME-iDS is not properly installed." + Environment.NewLine);
                textBoxInfo.AppendText(ex.Message + Environment.NewLine);
            }
        }

        private void GetSubdevicesWithBitchange(int numDevices)
        {
            if ((openErrSuccess == meIDS.ME_ERRNO_SUCCESS) && (numDevices > 0))
            {
                textBoxInfo.AppendText("Retrieving sub-devices of type ME_TYPE_DI (Digital In) and ME_TYPE_DIO (Digital In/Out) and bitchange capability..." + Environment.NewLine);

                for (int idxDevice = 0; idxDevice < numDevices; idxDevice++)
                {
                    int plugged;
                    int meError = GetDevicePlugged(idxDevice, out plugged);
                    if (meError == meIDS.ME_ERRNO_SUCCESS)
                    {
                        if (plugged == meIDS.ME_PLUGGED_IN)
                        {
                            GetSubdevicesByType(idxDevice, meIDS.ME_TYPE_DI);  
                            GetSubdevicesByType(idxDevice, meIDS.ME_TYPE_DIO); 
                        }
                        else
                        {
                            textBoxInfo.AppendText("Device " + idxDevice.ToString() + " not plugged in" + Environment.NewLine);
                        }
                    }
                }
            }
            DisplaySubdevices();
        }

        private void DisplaySubdevices()
        {
            if (coBoxSubDevs.Items.Count > 0)
            {
                textBoxInfo.AppendText("Number of sub-devices with bitchange found: " + coBoxSubDevs.Items.Count.ToString() + Environment.NewLine);
                for (int i = 0; i < coBoxSubDevs.Items.Count; i++)
                {
                    textBoxInfo.AppendText(coBoxSubDevs.Items[i] + Environment.NewLine);  
                }
                textBoxInfo.AppendText(Environment.NewLine);
                btnResetSubdev.Enabled = true;
                coBoxSubDevs.SelectedIndex = 0;                             
                FillSubdeviceCaps(coBoxSubDevs.SelectedIndex);
                EnableUI(true);
            }
            else
            {
                textBoxInfo.AppendText("No sub-devices with bitchange capability found." + Environment.NewLine);  
                CloseMEiDSDriver();
            }
        }

        private void GetSubdevicesByType(int idxDevice, int subdeviceType)
        {
            int meError;
            int numSubdevices;
            int idxMatchedSubdevice;
            StringBuilder devName = new StringBuilder(meIDS.ME_DEVICE_NAME_MAX_COUNT); 
            meError = meIDS.meQueryNumberSubdevices(idxDevice, out numSubdevices); 
            if (meError == meIDS.ME_ERRNO_SUCCESS)
            {
                int idxSubdevice = 0;
                meIDS.meQueryNameDevice(idxDevice, devName, devName.Capacity);  
                while (idxSubdevice < numSubdevices)
                {
                    meError = meIDS.meQuerySubdeviceByType(idxDevice,                  
                                                           idxSubdevice,             
                                                           subdeviceType,                    
                                                           meIDS.ME_SUBTYPE_ANY,     
                                                           out idxMatchedSubdevice); 
                    if (meError == meIDS.ME_ERRNO_SUCCESS)
                    {
                        AddBitchangeSubdevice(devName.ToString(), idxDevice, idxMatchedSubdevice, subdeviceType);
                        idxSubdevice = idxMatchedSubdevice + 1;                 
                    }
                    else
                    {
                        break;                                                  
                    }
                }
            }
        }

        private void AddBitchangeSubdevice(String devName, int idxDevice, int idxSubdevice, int subdeviceType)
        {
            int subdevCaps;
            int desiredCaps;
            desiredCaps = GetBitChangeCaps();           
            int meError = meIDS.meQuerySubdeviceCaps(idxDevice,           
                                                 idxSubdevice, 
                                                 out subdevCaps);   
            if ((meError == meIDS.ME_ERRNO_SUCCESS) && ((subdevCaps & desiredCaps) != 0))
            {
                int numOfChannels;
                meIDS.meQueryNumberChannels(idxDevice, idxSubdevice, out numOfChannels);  
                BitChSubdevice bitChSubdev = new BitChSubdevice(devName.ToString(), idxDevice, idxSubdevice, numOfChannels, subdevCaps);
                subdeviceList.Add(bitChSubdev);                     

                coBoxSubDevs.Items.Add(devName.ToString()      
                                        + ",  Device " + idxDevice.ToString()
                                        + ",  Subdevice " + idxSubdevice.ToString()
                                        + ",  Type: " + GetSubDevTypeName(subdeviceType)
                                        + ",  Channels: " + numOfChannels.ToString());
            }
        }

        private static int GetBitChangeCaps()
        {
            int desiredCaps = meIDS.ME_CAPS_DIO_BIT_PATTERN_IRQ
                          | meIDS.ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_RISING
                          | meIDS.ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_FALLING
                          | meIDS.ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_ANY;
            return desiredCaps;
        }

        private void FillSubdeviceCaps(int idxSelectedSubdev)
        {
            if(idxSelectedSubdev >= 0)
            {
                BitChSubdevice bitChSubdev = subdeviceList[idxSelectedSubdev];
                coBoxEdges.Items.Clear();
                if ((bitChSubdev.subdevCaps & meIDS.ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_RISING) == meIDS.ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_RISING)
                {
                    coBoxEdges.Items.Add(new ComboBoxItem("Rising", meIDS.ME_IRQ_EDGE_RISING));
                }
                if ((bitChSubdev.subdevCaps & meIDS.ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_FALLING) == meIDS.ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_FALLING)
                {
                    coBoxEdges.Items.Add(new ComboBoxItem("Falling", meIDS.ME_IRQ_EDGE_FALLING));
                }
                if ((bitChSubdev.subdevCaps & meIDS.ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_ANY) == meIDS.ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_ANY)
                {
                    coBoxEdges.Items.Add(new ComboBoxItem("Any", meIDS.ME_IRQ_EDGE_ANY));
                }
                coBoxEdges.SelectedIndex = coBoxEdges.Items.Count - 1;

                UncheckBitMask();
                ClearBitIndicators();
            }
        }

        private void EnableUI(bool isEnabled)
        {
            btnStartIRQ.Enabled = isEnabled;
            btnStopIRQ.Enabled = !isEnabled;
            coBoxSubDevs.Enabled = isEnabled;
            grpBoxBitMask.Enabled = isEnabled;
            btnSelectAll.Enabled = isEnabled;
            btnSelectNone.Enabled = isEnabled;
            btnSelectInvert.Enabled = isEnabled;
            btnSelectUpperBits.Enabled = isEnabled;
            btnSelectLowerBits.Enabled = isEnabled;
            coBoxEdges.Enabled = isEnabled;
            coBoxStatusFlag.Enabled = isEnabled;
        }

        private void btnResetSubdev_Click(object sender, EventArgs e)
        {
            BitChSubdevice bitChSubdev;
            bitChSubdev = subdeviceList[coBoxSubDevs.SelectedIndex];
            int meError = meIDS.meIOResetSubdevice(bitChSubdev.deviceIndex,         
                                                   bitChSubdev.subdevIndex,         
                                                   meIDS.ME_IO_RESET_SUBDEVICE_NO_FLAGS); 
            if (meError == meIDS.ME_ERRNO_SUCCESS)
            {
                textBoxInfo.AppendText("Reset Device " + bitChSubdev.deviceIndex.ToString() 
                    + ", Subdevice " + bitChSubdev.subdevIndex.ToString() + Environment.NewLine);
            }
            else
            {
                DisplayErrorMessage("Error reset Subdevice", meError);
            }
            ClearBitIndicators();
        }

        private void coBoxSubDevs_SelectedIndexChanged(object sender, EventArgs e)
        {
            FillSubdeviceCaps(coBoxSubDevs.SelectedIndex);
        }

        private void btnStartIRQ_Click(object sender, EventArgs e)
        {
            if (openErrSuccess == meIDS.ME_ERRNO_SUCCESS && coBoxSubDevs.SelectedIndex >= 0)
            {
                int meError;
                BitChSubdevice bitChSubdev = subdeviceList[coBoxSubDevs.SelectedIndex];
                
                EnableUI(false);
                try
                {
                    ConfigureSubdeviceForInput(bitChSubdev);
                    if (rbUseCallbackFunc.Checked)
                    {
                        SetIrqCallbackFunction(bitChSubdev);
                        meError = StartIrq(bitChSubdev);
                        if (meError != meIDS.ME_ERRNO_SUCCESS)
                        {
                            DisplayErrorMessage("meIOIrqStart Error:", meError);
                            EnableUI(true);
                        }
                    }
                    else if (rbUseIRQWaitFunc.Checked)
                    {
                        if (objectHandle.IsAllocated)
                        {
                            objectHandle.Free();
                            RemoveCallbackFunction(bitChSubdev);
                        }
                        
                        int iTimeout = GetTimeoutValue();
                        meError = StartIrq(bitChSubdev);              
                        if (meError == meIDS.ME_ERRNO_SUCCESS)
                        {
                            bitChSubdev.timeout = iTimeout;
                            backgroundWorker1.RunWorkerAsync(bitChSubdev); 
                        }
                        if (meError != meIDS.ME_ERRNO_SUCCESS)
                        {
                            DisplayErrorMessage("Error:", meError);
                        }
                    }
                }
                catch (System.ArgumentException aEx)
                {
                    MessageBox.Show("Error: " + aEx.Message);
                    EnableUI(true);
                }
                catch (System.Exception ex)
                {
                    MessageBox.Show("Error: " + ex.Message);
                    EnableUI(true);
                }
            }
        }

        private int StartIrq(BitChSubdevice bitChSubdev)
        {
            int meError;
            int irqArg;
            int irqEdge;
            int irqStatusFlag;
            irqArg = GetEdgesMask();
            irqEdge = ((ComboBoxItem)(coBoxEdges.SelectedItem)).GetValue;
            irqStatusFlag = ((ComboBoxItem)(coBoxStatusFlag.SelectedItem)).GetValue;
            DisplayEdgesAndBitMask(irqArg);
            meError = meIDS.meIOIrqStart(bitChSubdev.deviceIndex,
                                                 bitChSubdev.subdevIndex,
                                                 0,
                                                 meIDS.ME_IRQ_SOURCE_DIO_MASK,
                                                 irqEdge,
                                                 irqArg,
                                                 irqStatusFlag);
            return meError;
        }

        private void SetIrqCallbackFunction(BitChSubdevice bitChSubdev)
        {
            if (!objectHandle.IsAllocated)
            {
                objectHandle = GCHandle.Alloc(this);
            }
            meIDS.meIOIrqSetCallback(bitChSubdev.deviceIndex,
                                     bitChSubdev.subdevIndex,
                                     IrqCallback,
                                     (IntPtr)objectHandle,
                                     meIDS.ME_IO_IRQ_SET_CALLBACK_NO_FLAGS);
        }

        private void DisplayEdgesAndBitMask(int irqArg)
        {
            textBoxInfo.AppendText("StartIRQ set Edges '" + ((ComboBoxItem)(coBoxEdges.SelectedItem)).ToString()
                 + "', BitMask: " + IntToBin(irqArg, 32) + Environment.NewLine);
        }

        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            int value;
            int irqCount;
            int meError;
            BitChSubdevice bitChSubdev = e.Argument as BitChSubdevice;
            bitChSubdev.meErrorCode = meIDS.ME_ERRNO_SUCCESS;

            while (true)
            {
                meError = meIDS.meIOIrqWait(bitChSubdev.deviceIndex,
                                            bitChSubdev.subdevIndex,
                                            0,
                                            out irqCount,
                                            out value,
                                            bitChSubdev.timeout,
                                            meIDS.ME_IO_IRQ_WAIT_NO_FLAGS);
                if (meError == meIDS.ME_ERRNO_SUCCESS)
                {
                    DisplayIrqValues(bitChSubdev.deviceIndex, bitChSubdev.subdevIndex, 0, irqCount, value, IntPtr.Zero, 0);
                }
                else
                {
                    bitChSubdev.meErrorCode = meError;
                    break;
                }
            }

            meIDS.meIOIrqStop(bitChSubdev.deviceIndex,
                              bitChSubdev.subdevIndex,
                              0,
                              meIDS.ME_IO_IRQ_STOP_NO_FLAGS);
            e.Result = bitChSubdev;
        }

        private void backgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            BitChSubdevice bitChSubdev = e.Result as BitChSubdevice;
            EnableUI(true);
            if (bitChSubdev.meErrorCode != meIDS.ME_ERRNO_SUCCESS)
            {
                DisplayErrorMessage("meIOIrqWait Error: ", bitChSubdev.meErrorCode);
                bitChSubdev.meErrorCode = meIDS.ME_ERRNO_SUCCESS;
            }
        }

        private int GetTimeoutValue()
        {
            int iTimeout = int.Parse(numUDTimeout.Text);
            return iTimeout;
        }

        private void ConfigureSubdeviceForInput(BitChSubdevice bitChSubdev)
        {
            int meError = meIDS.meIOSingleConfig(bitChSubdev.deviceIndex,
                                                 bitChSubdev.subdevIndex,
                                                 0,
                                                 meIDS.ME_SINGLE_CONFIG_DIO_INPUT,
                                                 meIDS.ME_REF_NONE,
                                                 meIDS.ME_TRIG_CHAN_NONE,                          
                                                 meIDS.ME_TRIG_TYPE_NONE,
                                                 meIDS.ME_TRIG_EDGE_NONE,
                                                 bitChSubdev.singleConfigFlag

                );
        }

        private void btnStopIRQ_Click(object sender, EventArgs e)
        {
            StopIrq();
        }

        private void StopIrq()
        {
            if (openErrSuccess == meIDS.ME_ERRNO_SUCCESS && coBoxSubDevs.SelectedIndex >= 0)
            {
                BitChSubdevice bitChSubdev = subdeviceList[coBoxSubDevs.SelectedIndex];
                meIDS.meIOIrqStop(bitChSubdev.deviceIndex, bitChSubdev.subdevIndex, 0, meIDS.ME_IO_IRQ_STOP_NO_FLAGS); 
                EnableUI(true);
                textBoxInfo.AppendText("Stop IRQ on Device " + bitChSubdev.deviceIndex.ToString() + ", Subdevice " + bitChSubdev.subdevIndex.ToString() + Environment.NewLine);
            }
            ClearBitIndicators();
        }

        // This delegate enables asynchronous calls for setting the text in textBoxInfo and updating the up-, down-arrow bitmaps in lvBitMask
        private delegate void DisplayIrqValuesDelegate(int idxDevice, int idxSubdevice, int idxChannel,
                                            int irqCount, int value, IntPtr context, int errorCode);

        // This method demonstrates a pattern for making thread-safe calls on a Windows Forms control.
        private void DisplayIrqValues(int idxDevice, int idxSubdevice, int idxChannel,
                                            int irqCount, int value, IntPtr context, int errorCode)
        {
            // InvokeRequired compares the thread ID of the calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (this.InvokeRequired)
            {
                //The calling thread is different from the thread that created the TextBox and ListView controls.
                //The method creates a SetTextCallback and calls itself asynchronously using the Invoke method.
                DisplayIrqValuesDelegate dispDelegate = new DisplayIrqValuesDelegate(DisplayIrqValues);
                this.Invoke(dispDelegate, new object[] {idxDevice,  idxSubdevice, idxChannel, irqCount, value, context, errorCode});
            }
            else
            {
                //The calling thread is the same as the thread that created the TextBox and ListView controls
                //The Text property and the ListView items are set directly.
                DisplayValues(idxDevice, idxSubdevice, value);
            }
        }

        private void DisplayValues(int idxDevice, int idxSubdevice, int value)
        {
            dataLine++;
            textBoxInfo.AppendText(String.Format("{0:00}", dataLine)
                        + " Bitchange on Device: " + idxDevice.ToString()
                        + ",  Subdevice: " + idxSubdevice.ToString()
                        + ",  Value hex: " + String.Format("0x{0:X}", value)
                        + ",  bin: " + IntToBin(value, 32)
                        + ",  dec: " + value.ToString()
                        + Environment.NewLine);

            DisplayBitIndicators(value);
        }

        private static int meIOIrqCB(int idxDevice, int idxSubdevice, int idxChannel,
                                            int irqCount, int value, IntPtr pContext, int errorCode)
        {
            if (errorCode == meIDS.ME_ERRNO_SUCCESS && pContext != IntPtr.Zero)
            {
                GCHandle objHandle = GCHandle.FromIntPtr(pContext);          
                frmMain fm = (frmMain)objHandle.Target;
                if (fm is BitChange.frmMain)
                {
                    fm.DisplayIrqValues(idxDevice, idxSubdevice, idxChannel, irqCount, value, pContext, errorCode);  
                }
            }
            //if a value not equal to zero is returned the operation will stop. 
            //The easiest way to do this is just return the given error code
            return errorCode; 
        }

        private void RemoveCallbackFunction(BitChSubdevice bitChSubdev)
        {
            meIDS.meIOIrqSetCallback(bitChSubdev.deviceIndex,
                                    bitChSubdev.subdevIndex,
                                    null,
                                    IntPtr.Zero,
                                    meIDS.ME_IO_IRQ_SET_CALLBACK_NO_FLAGS);
        }

        private int GetEdgesMask()
        {
            int irqArg = 0;
            for (int i = 31; i >= 0; i--)
            {
                if (bitChBoxes[i].Checked)
                    irqArg |= (1 << i);
            }
            if (irqArg == 0)
            {
                throw new System.ArgumentException("Please check the bits in the Edges Bit Mask that will detect a bit change.");
            }

            return irqArg;
        }

        private void btnSelectAll_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < 32; i++)
            {
                bitChBoxes[i].Checked = true;
            }
        }

        private void btnSelectNone_Click(object sender, EventArgs e)
        {
            UncheckBitMask();
        }

        private void UncheckBitMask()
        {
            for (int i = 0; i < 32; i++)
            {
                bitChBoxes[i].Checked = false;
            }
        }

        private void btnSelectInvert_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < 32; i++)
            {
                bitChBoxes[i].Checked = !bitChBoxes[i].Checked;
            }
        }

        private void btnSelectLowerBits_Click(object sender, EventArgs e)
        {
            if (coBoxSubDevs.SelectedIndex >= 0)
            {
                BitChSubdevice bitChSubdev = subdeviceList[coBoxSubDevs.SelectedIndex];
                int numBits = bitChSubdev.numOfChannels;
                if (numBits > 16)
                {
                    numBits = 16;
                }
                for (int i = 0; i < numBits; i++)
                {
                    bitChBoxes[i].Checked = true;
                }
            }
        }

        private void btnSelectUpperBits_Click(object sender, EventArgs e)
        {
            if (coBoxSubDevs.SelectedIndex >= 0)
            {
                BitChSubdevice bitChSubdev = subdeviceList[coBoxSubDevs.SelectedIndex];
                int numBits = bitChSubdev.numOfChannels;
                if (numBits > 16)
                {
                    numBits = 16;
                }
                for (int i = 16; i < 16 + numBits; i++)
                {
                    bitChBoxes[i].Checked = true;
                }
            }
        }

        private void btnClearIndicator_Click(object sender, EventArgs e)
        {
            ClearBitIndicators();
        }

        private void ClearBitIndicators()
        {
            for (int i = 0; i < 32; i++)
            {
                bitIndicators[i].BackColor = System.Drawing.Color.White;
            }
        }

        private void InitBitMaskCheckBoxArray()
        {
            bitChBoxes[0] = chBoxBit0;
            bitChBoxes[1] = chBoxBit1;
            bitChBoxes[2] = chBoxBit2;
            bitChBoxes[3] = chBoxBit3;
            bitChBoxes[4] = chBoxBit4;
            bitChBoxes[5] = chBoxBit5;
            bitChBoxes[6] = chBoxBit6;
            bitChBoxes[7] = chBoxBit7;
            bitChBoxes[8] = chBoxBit8;
            bitChBoxes[9] = chBoxBit9;
            bitChBoxes[10] = chBoxBit10;
            bitChBoxes[11] = chBoxBit11;
            bitChBoxes[12] = chBoxBit12;
            bitChBoxes[13] = chBoxBit13;
            bitChBoxes[14] = chBoxBit14;
            bitChBoxes[15] = chBoxBit15;
            bitChBoxes[16] = chBoxBit16;
            bitChBoxes[17] = chBoxBit17;
            bitChBoxes[18] = chBoxBit18;
            bitChBoxes[19] = chBoxBit19;
            bitChBoxes[20] = chBoxBit20;
            bitChBoxes[21] = chBoxBit21;
            bitChBoxes[22] = chBoxBit22;
            bitChBoxes[23] = chBoxBit23;
            bitChBoxes[24] = chBoxBit24;
            bitChBoxes[25] = chBoxBit25;
            bitChBoxes[26] = chBoxBit26;
            bitChBoxes[27] = chBoxBit27;
            bitChBoxes[28] = chBoxBit28;
            bitChBoxes[29] = chBoxBit29;
            bitChBoxes[30] = chBoxBit30;
            bitChBoxes[31] = chBoxBit31;
        }

        private void InitBitIndicatorsArray()
        {
            bitIndicators[0] = lblBit0;
            bitIndicators[1] = lblBit1;
            bitIndicators[2] = lblBit2;
            bitIndicators[3] = lblBit3;
            bitIndicators[4] = lblBit4;
            bitIndicators[5] = lblBit5;
            bitIndicators[6] = lblBit6;
            bitIndicators[7] = lblBit7;
            bitIndicators[8] = lblBit8;
            bitIndicators[9] = lblBit9;
            bitIndicators[10] = lblBit10;
            bitIndicators[11] = lblBit11;
            bitIndicators[12] = lblBit12;
            bitIndicators[13] = lblBit13;
            bitIndicators[14] = lblBit14;
            bitIndicators[15] = lblBit15;
            bitIndicators[16] = lblBit16;
            bitIndicators[17] = lblBit17;
            bitIndicators[18] = lblBit18;
            bitIndicators[19] = lblBit19;
            bitIndicators[20] = lblBit20;
            bitIndicators[21] = lblBit21;
            bitIndicators[22] = lblBit22;
            bitIndicators[23] = lblBit23;
            bitIndicators[24] = lblBit24;
            bitIndicators[25] = lblBit25;
            bitIndicators[26] = lblBit26;
            bitIndicators[27] = lblBit27;
            bitIndicators[28] = lblBit28;
            bitIndicators[29] = lblBit29;
            bitIndicators[30] = lblBit30;
            bitIndicators[31] = lblBit31;
        }

        private void FillStatusFlags()
        {
            coBoxStatusFlag.Items.Add(new ComboBoxItem("Standard", meIDS.ME_IO_IRQ_START_NO_FLAGS));
            coBoxStatusFlag.Items.Add(new ComboBoxItem("Extended", meIDS.ME_IO_IRQ_START_EXTENDED_STATUS));
            coBoxStatusFlag.SelectedIndex = 0;
        }

        private string MeVersionToString(int version)
        {
            return String.Format("{0}.{1}.{2}.{3}", version >> 24, 0xFF & (version >> 16), 0xFF & (version >> 8), 0xFF & version);
        }

        private string GetSubDevTypeName(int subdeviceType)
        {
            string tmpType;
            switch (subdeviceType)
            {
                case meIDS.ME_TYPE_INVALID:
                    tmpType = "ME_TYPE_INVALID";
                    break;
                case meIDS.ME_TYPE_AI:
                    tmpType = "ME_TYPE_AI";
                    break;
                case meIDS.ME_TYPE_AO:
                    tmpType = "ME_TYPE_AO";
                    break;
                case meIDS.ME_TYPE_DIO:
                    tmpType = "ME_TYPE_DIO";
                    break;
                case meIDS.ME_TYPE_DO:
                    tmpType = "ME_TYPE_DO";
                    break;
                case meIDS.ME_TYPE_DI:
                    tmpType = "ME_TYPE_DI";
                    break;
                case meIDS.ME_TYPE_CTR:
                    tmpType = "ME_TYPE_CTR";
                    break;
                case meIDS.ME_TYPE_EXT_IRQ:
                    tmpType = "ME_TYPE_EXT_IRQ";
                    break;
                case meIDS.ME_TYPE_FIO:
                    tmpType = "ME_TYPE_FIO";
                    break;
                case meIDS.ME_TYPE_FO:
                    tmpType = "ME_TYPE_FO";
                    break;
                case meIDS.ME_TYPE_FI:
                    tmpType = "ME_TYPE_FI";
                    break;
                case meIDS.ME_TYPE_FPGA:
                    tmpType = "ME_TYPE_FPGA";
                    break;
                default:
                    tmpType = String.Format("0x{0:X}", subdeviceType);
                    break;
            };
            return tmpType;
        }

        private string IntToBin(int val, int dispDigits)
        {
            int b;
            bool found = false;
            string tmpStr = "";
            for (b = 0; b < Constants.DIGITNUMS.Length; b++)
            {
                if (dispDigits == Constants.DIGITNUMS[b])
                {
                    found = true;
                    break;
                }
            }
            if (found)
            {
                if (dispDigits == 1)
                {
                    tmpStr = (val & 1).ToString();
                }
                else
                {
                    for (int i = (dispDigits >> 2) - 1; i >= 0; i--)
                    {
                        b = (val >> (i << 2)) & 0x0F;
                        tmpStr += Constants.BIN_STR[b];
                    }
                }
            }
            return tmpStr;
        }

        private void DisplayBitIndicators(int bitValue)
        {
            int bitPos = 1;
            for (int i = 0; i < 32; i++)
            {
                if ((bitValue & bitPos) == bitPos)
                {
                    bitIndicators[i].BackColor = System.Drawing.Color.Green;
                }
                else
                {
                    bitIndicators[i].BackColor = System.Drawing.Color.White;
                }
                bitPos <<= 1;
            }
        }

        private void DisplayErrorMessage(string message, int meError)
        {
            StringBuilder errMessage = new StringBuilder(meIDS.ME_ERROR_MSG_MAX_COUNT);   
            textBoxInfo.AppendText(message + meError.ToString() + Environment.NewLine);
            meIDS.meErrorGetMessage(meError, errMessage, errMessage.Capacity);            
            textBoxInfo.AppendText("ErrorMessage: " + errMessage + Environment.NewLine);  
        }

        private int GetDevicePlugged(int idxDevice, out int plugged)
        {
            int vendorId;
            int deviceId;
            int serialNo;
            int busType;
            int busNo;
            int devNo;
            int funcNo;


            int meError = meIDS.meQueryInfoDevice(idxDevice,            
                                             out vendorId,              
                                             out deviceId,              
                                             out serialNo,              
                                             out busType,               
                                             out busNo,                 
                                             out devNo,                 
                                             out funcNo,                
                                             out plugged);              //Pointer indicates whether a device is physically available
            return meError;
        }

        private void CloseMEiDSDriver()
        {
            if (openErrSuccess == meIDS.ME_ERRNO_SUCCESS)
            {
                if (coBoxSubDevs.SelectedIndex >= 0)
                {
                    BitChSubdevice bitChSubdev = subdeviceList[coBoxSubDevs.SelectedIndex];
                    meIDS.meIOIrqStop(bitChSubdev.deviceIndex, bitChSubdev.subdevIndex, 0, meIDS.ME_IO_IRQ_STOP_NO_FLAGS); 
                }
                if (objectHandle.IsAllocated)                   
                {
                    objectHandle.Free();                        // do this when callback cannot be called anymore
                }
                meIDS.meClose(meIDS.ME_CLOSE_NO_FLAGS);         
                openErrSuccess = Constants.ME_IDS_NOTOPENED;
                btnResetSubdev.Enabled = false;
                EnableUI(false);
                subdeviceList.Clear();
                coBoxSubDevs.Items.Clear();
                coBoxEdges.Items.Clear();
            }
        }

        private void DisplayVersionMainDriver()
        {
            int tmpVersion;
            meIDS.meQueryVersionMainDriver(out tmpVersion);
            textBoxInfo.AppendText("Maindriver version: " + MeVersionToString(tmpVersion) + Environment.NewLine + Environment.NewLine); 
        }

        private void DisplayVersionLibrary()
        {
            int tmpVersion;
            meIDS.meQueryVersionLibrary(out tmpVersion);
            textBoxInfo.AppendText("Library version: " + MeVersionToString(tmpVersion) + Environment.NewLine);
        }

        private void btnClearHwInfo_Click(object sender, EventArgs e)
        {
            textBoxInfo.Clear();
            dataLine = 0;
        }

        private void btnAbout_Click(object sender, EventArgs e)
        {
            AboutBox1 aboutBox = new AboutBox1();
            aboutBox.ShowDialog();
        }
    }

    class BitChSubdevice
    {
        public string deviceName;
        public int deviceIndex;            
        public int subdevIndex;            
        public int numOfChannels;          
        public int subdevCaps;
        public int singleConfigFlag;
        public int timeout;
        public int meErrorCode;

        public BitChSubdevice(string DeviceName, int DeviceIndex, int SubdevIndex, int NumOfChannels, int Capabilities)
        {
            this.deviceName = DeviceName;
            this.deviceIndex = DeviceIndex;
            this.subdevIndex = SubdevIndex;
            this.numOfChannels = NumOfChannels;
            this.subdevCaps = Capabilities;

            switch (NumOfChannels)
            {
                case 1: singleConfigFlag = meIDS.ME_IO_SINGLE_CONFIG_DIO_BIT;
                    break;
                case 8: singleConfigFlag = meIDS.ME_IO_SINGLE_CONFIG_DIO_BYTE;
                    break;
                case 16: singleConfigFlag = meIDS.ME_IO_SINGLE_CONFIG_DIO_WORD;
                    break;
                case 24: singleConfigFlag = meIDS.ME_IO_SINGLE_CONFIG_DIO_DWORD;
                    break;
                case 32: singleConfigFlag = meIDS.ME_IO_SINGLE_CONFIG_DIO_DWORD;
                    break;
                default: singleConfigFlag = meIDS.ME_VALUE_INVALID;
                    break;
            }
        }
    }

    public class ComboBoxItem
    {
        string dispValue;
        int val;

        public ComboBoxItem(string displayValue, int value)
        {
            dispValue = displayValue;
            val = value;
        }

        public override string ToString()
        {
            return dispValue;
        }

        public int GetValue
        {
            get
            {
                return val;
            }
        }
    }

    static class Constants
    {
        public const int ME_IDS_NOTOPENED = -1;                                 
        public static readonly string[] BIN_STR = {"0000", "0001", "0010", "0011",  
                                                   "0100", "0101", "0110", "0111",
                                                   "1000", "1001", "1010", "1011",
                                                   "1100", "1101", "1110", "1111"};
        public static int[] DIGITNUMS = { 1, 8, 16, 24, 32 };                
    }
}
