﻿//==============================================================================
//Copyright(c), Meilhaus Electronic GmbH
//              http://www.meilhaus.de
//              support@meilhaus.de
//
//Source File : uMain.pas
//Description : Example program for using subdevices with Counter 8254
//              - Open main driver (ME-iDS)
//              - Go through the list of all installed ME-iDS devices
//                and find subdevices of type Counter and subtype 8254
//              - 3 counters at once can be selected from the subdevice combobox
//                as the integrated circuit 82C54 always contains 3 counters
//              - The counters can be used independently
//              - On some boards (e.g. ME-1400) counters of the same component
//                can be cascaded without external connections.
//              - Button "Write preload": Writes the preload value to the counter subdevice
//                and stops the timer if enabled
//              - "Single Read": single read from selected subdevice (counter)
//              - "Write & Run": writes the preload value to the subdevice
//                and starts a timer that reads the counter value periodically
//              - "Stop": stops the timer
//              - "Reset": stop the timer and reset the subdevice
//              - Close main driver when program is closed
//==============================================================================

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using meIDS_CS;

namespace Counter8254
{
    public partial class frmMain : Form
    {
        private int openErrSuccess = Constants.ME_IDS_NOTOPENED;                
        private int lineNumber = 0;                                               
        private List<CounterSubdevice> subdeviceList = new List<CounterSubdevice>(); 

        public frmMain()
        {
            InitializeComponent();
        }

        private void frmMain_Shown(object sender, EventArgs e)
        {
            coBoxMode1.SelectedIndex = 0;
            coBoxMode2.SelectedIndex = 0;
            coBoxMode3.SelectedIndex = 0;

            OpenMEiDSDriver();
        }

        private void OpenMEiDSDriver()
        {
            try
            {
                textBoxInfo.AppendText("Opening Maindriver..." + Environment.NewLine);
                openErrSuccess = meIDS.meOpen(meIDS.ME_OPEN_NO_FLAGS);          

                if (openErrSuccess == meIDS.ME_ERRNO_SUCCESS)
                {
                    DisplayVersionLibrary();
                    DisplayVersionMainDriver();
                    textBoxInfo.AppendText(Environment.NewLine);

                    int numDevices;
                    meIDS.meQueryNumberDevices(out numDevices);                     
                    textBoxInfo.AppendText("Number of devices found: " + numDevices.ToString() + Environment.NewLine);

                    GetCounter8254Subdevices(numDevices);                           
                }
                else
                {
                    DisplayErrorMessage("meOpen - Error: ", openErrSuccess);
                }
            }
            catch (DllNotFoundException ex)
            {
                textBoxInfo.AppendText("The ME-iDS is not properly installed." + Environment.NewLine);
                textBoxInfo.AppendText(ex.Message + Environment.NewLine);
            }
        }

        private void GetCounter8254Subdevices(int numDevices)
        {
            if (openErrSuccess == meIDS.ME_ERRNO_SUCCESS && numDevices > 0) 
            {
                textBoxInfo.AppendText("Retrieving sub-devices of type ME_TYPE_CTR (Counter), subtype ME_SUBTYPE_CTR_8254 (8254)..." + Environment.NewLine);
                for (int idxDevice = 0; idxDevice < numDevices; idxDevice++)
                {
                    int plugged;
                    int meError = GetDevicePlugged(idxDevice, out plugged);

                    if (meError == meIDS.ME_ERRNO_SUCCESS)
                    {
                        if (plugged == meIDS.ME_PLUGGED_IN)
                        {
                            GetSubdevicesByType(idxDevice, meIDS.ME_TYPE_CTR, meIDS.ME_SUBTYPE_CTR_8254); 
                        }
                        else
                        {
                            textBoxInfo.AppendText("Device " + idxDevice.ToString() + " not plugged in" + Environment.NewLine);
                        }
                    }
                    else
                    {
                        DisplayErrorMessage("meQueryInfoDevice Error: ", meError);
                    }
                }

            }
            DisplaySubdevices();
        }

        private void GetSubdevicesByType(int idxDevice, int subdeviceType, int subdeviceSubtype)
        {
            int meError;
            int numSubdevices;

            meError = meIDS.meQueryNumberSubdevices(idxDevice, out numSubdevices); 
            if (meError == meIDS.ME_ERRNO_SUCCESS)
            {
                int idxMatchedSubdevice;
                int numMatched = 0;
                int idxSubdevice = 0;
                StringBuilder devName = new StringBuilder(meIDS.ME_DEVICE_NAME_MAX_COUNT); 
                
                meIDS.meQueryNameDevice(idxDevice, devName, devName.Capacity);  
                while (idxSubdevice < numSubdevices)
                {
                    meError = meIDS.meQuerySubdeviceByType(idxDevice,                
                                                           idxSubdevice,             
                                                           subdeviceType,            
                                                           meIDS.ME_SUBTYPE_ANY,     
                                                           out idxMatchedSubdevice); 
                    if (meError == meIDS.ME_ERRNO_SUCCESS)
                    {
                        numMatched++;
                        AddCountersText(devName.ToString(), idxDevice, idxMatchedSubdevice, numMatched);
                        AddCounterSubdevice(devName.ToString(), idxDevice, idxMatchedSubdevice);
                        idxSubdevice = idxMatchedSubdevice + 1;                 
                    }
                    else
                    {
                        break;                                                  
                    }
                }
            }
        }

        private void AddCountersText(String devName, int idxDevice, int idxSubdevice, int numMatched)
        {
            if (((numMatched - 1) % 3) == 2)
            {
                string countersTxt;
                countersTxt = devName.ToString()
                            + ", Device " + idxDevice.ToString()
                            + String.Format(", Subdevices {0}, {1}, {2}", idxSubdevice - 2, idxSubdevice - 1, idxSubdevice);
                coBoxSubDevs.Items.Add(countersTxt);
            }
        }

        private void AddCounterSubdevice(String devName, int idxDevice, int idxSubdevice)
        {
            int subdevCaps;
            int args = 0;
            meIDS.meQuerySubdeviceCaps(idxDevice, idxSubdevice, out subdevCaps);
            meIDS.meQuerySubdeviceCapsArgs(idxDevice, idxSubdevice, meIDS.ME_CAP_CTR_WIDTH, ref args, 1);

            CounterSubdevice ctrSubdevice = new CounterSubdevice(devName, idxDevice, idxSubdevice, subdevCaps, (1 << args) - 1);
            subdeviceList.Add(ctrSubdevice);
        }

        private void coBoxSubDevs_SelectedIndexChanged(object sender, EventArgs e)
        {
            ChangeCounterSubdevices(coBoxSubDevs.SelectedIndex);
        }

        private void ChangeCounterSubdevices(int idx)
        {
            if (idx >= 0)
            {
                StopAllTimers();

                CounterSubdevice ctrSubdevice = subdeviceList[idx * 3];
                coBoxMode1.SelectedIndex = 0;                                     
                FillCounterSources(ctrSubdevice, coBoxClockSource1);            
                coBoxPreloadValue1.Text = ctrSubdevice.counterMaxValue.ToString(); 
                editCtrValue1.Text = "0";
                grpBoxCounter1.Text = String.Format("Subdevice {0}", ctrSubdevice.subdevIndex);

                coBoxMode2.SelectedIndex = 0;                                   
                ctrSubdevice = subdeviceList[idx * 3 +1];                       
                FillCounterSources(ctrSubdevice, coBoxClockSource2);            
                coBoxPreloadValue2.Text = ctrSubdevice.counterMaxValue.ToString(); 
                editCtrValue2.Text = "0";
                grpBoxCounter2.Text = String.Format("Subdevice {0}", ctrSubdevice.subdevIndex);

                coBoxMode3.SelectedIndex = 0;                                   
                ctrSubdevice = subdeviceList[idx * 3 +2];                       
                FillCounterSources(ctrSubdevice, coBoxClockSource3);            
                coBoxPreloadValue3.Text = ctrSubdevice.counterMaxValue.ToString();
                editCtrValue3.Text = "0";
                grpBoxCounter3.Text = String.Format("Subdevice {0}", ctrSubdevice.subdevIndex);
            }
        }

        private void FillCounterSources(CounterSubdevice ctrSubdevice, ComboBox coBox)
        {
            coBox.Items.Clear();
            if ((ctrSubdevice.subdevCaps & meIDS.ME_CAPS_CTR_CLK_EXTERNAL) == meIDS.ME_CAPS_CTR_CLK_EXTERNAL)
            {
                coBox.Items.Add(new ComboBoxItem("External", meIDS.ME_REF_CTR_EXTERNAL));
            }
            
            if ((ctrSubdevice.subdevCaps & meIDS.ME_CAPS_CTR_CLK_PREVIOUS) == meIDS.ME_CAPS_CTR_CLK_PREVIOUS)
            {
                coBox.Items.Add(new ComboBoxItem("Previous", meIDS.ME_REF_CTR_PREVIOUS));
            }

            if ((ctrSubdevice.subdevCaps & meIDS.ME_CAPS_CTR_CLK_INTERNAL_1MHZ) == meIDS.ME_CAPS_CTR_CLK_INTERNAL_1MHZ)
            {
                coBox.Items.Add(new ComboBoxItem("Internal 1 MHz", meIDS.ME_REF_CTR_INTERNAL_1MHZ));
            }

            if ((ctrSubdevice.subdevCaps & meIDS.ME_CAPS_CTR_CLK_INTERNAL_10MHZ) == meIDS.ME_CAPS_CTR_CLK_INTERNAL_10MHZ)
            {
                coBox.Items.Add(new ComboBoxItem("Internal 10 MHz", meIDS.ME_REF_CTR_INTERNAL_10MHZ));
            }

            if (coBox.Items.Count > 0) coBox.SelectedIndex = 0;
        }

        private int SingleReadCounter(int ctrIdx, TextBox editValue)
        {
            int meError = meIDS.ME_ERRNO_SUCCESS;

            if (coBoxSubDevs.SelectedIndex >= 0)
            {
                int value = 0;
                meError = ReadCounterValue(ctrIdx, ref value);

                if (meError == meIDS.ME_ERRNO_SUCCESS)
                {
                    editValue.Text = value.ToString();            
                } 
                else
                {
                    editValue.Text = "Error " + meError.ToString();             
                }
            }
            return meError;
        }

        private int ReadCounterValue(int ctrIdx, ref int value)
        {
            CounterSubdevice ctrSubdevice = subdeviceList[coBoxSubDevs.SelectedIndex * 3 + ctrIdx];
            meIDS.meIOSingle_t[] io_single = new meIDS.meIOSingle_t[1];
            
            io_single[0].iDevice = ctrSubdevice.deviceIndex;
            io_single[0].iSubdevice = ctrSubdevice.subdevIndex;
            io_single[0].iChannel = 0;
            io_single[0].iDir = meIDS.ME_DIR_INPUT;
            io_single[0].iValue = 0;
            io_single[0].iTimeOut = 0;
            io_single[0].iFlags = meIDS.ME_IO_SINGLE_TYPE_NO_FLAGS;
            io_single[0].iErrno = 0;

            int meError = meIDS.meIOSingle(ref io_single[0],
                                       1,
                                       meIDS.ME_IO_SINGLE_NO_FLAGS);
            value = io_single[0].iValue;
            return meError;
        }

        private bool SingleWriteCounter(int ctrIdx)
        {
            bool result = false;
            if (coBoxSubDevs.SelectedIndex >= 0)
            {
                CounterSubdevice ctrSubdevice = subdeviceList[coBoxSubDevs.SelectedIndex * 3 + ctrIdx];
                int ctrMode;
                string ctrModeStr;
                int preloadVal;
                int ctrClockSource;
                string ctrClockSrcStr;
                int meError;

                try
                {
                    switch (ctrIdx)
                    {
                        case 0:
                            preloadVal = int.Parse(coBoxPreloadValue1.Text);
                            AddCoBoxValue(coBoxPreloadValue1, coBoxPreloadValue1.Text);
                            ctrMode = Constants.Ctr8254Modes[coBoxMode1.SelectedIndex];
                            ctrModeStr = coBoxMode1.Text;
                            ctrClockSource = ((ComboBoxItem)(coBoxClockSource1.SelectedItem)).GetValue;
                            ctrClockSrcStr = coBoxClockSource1.Text;
                            break;
                        case 1:
                            preloadVal = int.Parse(coBoxPreloadValue2.Text);
                            AddCoBoxValue(coBoxPreloadValue2, coBoxPreloadValue2.Text);
                            ctrMode = Constants.Ctr8254Modes[coBoxMode2.SelectedIndex];
                            ctrModeStr = coBoxMode2.Text;
                            ctrClockSource = ((ComboBoxItem)(coBoxClockSource2.SelectedItem)).GetValue;
                            ctrClockSrcStr = coBoxClockSource2.Text;
                            break;
                        case 2:
                            preloadVal = int.Parse(coBoxPreloadValue3.Text);
                            AddCoBoxValue(coBoxPreloadValue3, coBoxPreloadValue3.Text);
                            ctrMode = Constants.Ctr8254Modes[coBoxMode3.SelectedIndex];
                            ctrModeStr = coBoxMode3.Text;
                            ctrClockSource = ((ComboBoxItem)(coBoxClockSource3.SelectedItem)).GetValue;
                            ctrClockSrcStr = coBoxClockSource3.Text;
                            break;
                        default:
                            return false;
                    }

                    if (preloadVal < 0 || preloadVal > ctrSubdevice.counterMaxValue)
                    {
                        throw new System.ArgumentOutOfRangeException("Preload value is out of range. Allowed values are from 0..." 
                            + ctrSubdevice.counterMaxValue.ToString());
                    }

                    meError = ConfigureCounterSubdevice(ctrSubdevice, ctrMode, ctrClockSource);

                    if (meError != meIDS.ME_ERRNO_SUCCESS)
                    {
                        DisplayErrorMessage("meIOSingleConfig - Error: ", meError);
                        return false;
                    }

                    meError = WritePreloadValue(ctrSubdevice, preloadVal);
                    if (meError == meIDS.ME_ERRNO_SUCCESS)
                    {
                        DisplayValues(ctrSubdevice, ctrModeStr, ctrClockSrcStr, preloadVal);
                        result = true;
                    }
                    else
                    {
                        DisplayErrorMessage("meIOSingle - Error: ", meError);
                    }
                }
                catch (System.ArgumentOutOfRangeException argEx)
                {
                    MessageBox.Show(argEx.Message, this.Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (System.FormatException formatEx)
                {
                    MessageBox.Show(formatEx.Message, this.Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            return result;
        }

        private void DisplayValues(CounterSubdevice ctrSubdevice, string ctrModeStr, string ctrClockSrcStr, int preloadVal)
        {
            lineNumber++;
            textBoxInfo.AppendText(String.Format("{0:00}", lineNumber)
                   + " Write to " + ctrSubdevice.deviceName
                   + ",  Device: " + ctrSubdevice.deviceIndex.ToString()
                   + ",  Subdevice: " + ctrSubdevice.subdevIndex.ToString()
                   + ",  Counter mode: " + ctrModeStr
                   + ",  Clock source: " + ctrClockSrcStr
                   + ",  Preload value: " + preloadVal.ToString()
                   + Environment.NewLine);
        }

        private int ConfigureCounterSubdevice(CounterSubdevice ctrSubdevice, int ctrMode, int ctrClockSource)
        {
            int meError;
            meError = meIDS.meIOSingleConfig(ctrSubdevice.deviceIndex,      
                                            ctrSubdevice.subdevIndex,       
                                            0,                              
                                            ctrMode,                        
                                            ctrClockSource,                 
                                            meIDS.ME_TRIG_CHAN_DEFAULT,     
                                            meIDS.ME_TRIG_TYPE_NONE,        
                                            meIDS.ME_VALUE_NOT_USED,        
                                            meIDS.ME_IO_SINGLE_CONFIG_NO_FLAGS); 
            return meError;
        }

        private int WritePreloadValue(CounterSubdevice ctrSubdevice, int preloadVal)
        {
            meIDS.meIOSingle_t[] io_single = new meIDS.meIOSingle_t[1];
            io_single[0].iDevice = ctrSubdevice.deviceIndex;
            io_single[0].iSubdevice = ctrSubdevice.subdevIndex;
            io_single[0].iChannel = 0;
            io_single[0].iDir = meIDS.ME_DIR_OUTPUT;
            io_single[0].iValue = preloadVal;
            io_single[0].iTimeOut = 0;
            io_single[0].iFlags = meIDS.ME_IO_SINGLE_TYPE_NO_FLAGS;
            io_single[0].iErrno = 0;

            int meError = meIDS.meIOSingle(ref io_single[0],
                                        1,
                                        meIDS.ME_IO_SINGLE_NO_FLAGS);
            return meError;
        }

        private void ResetSubdevice(int ctrIdx)
        {
            if (coBoxSubDevs.SelectedIndex >= 0)
            {
                StopTimer(ctrIdx);
                CounterSubdevice ctrSubdevice = subdeviceList[coBoxSubDevs.SelectedIndex * 3 + ctrIdx];
                int meError = meIDS.meIOResetSubdevice(ctrSubdevice.deviceIndex,
                                              ctrSubdevice.subdevIndex,
                                              meIDS.ME_IO_RESET_SUBDEVICE_NO_FLAGS
                                              );
                if (meError == meIDS.ME_ERRNO_SUCCESS)
                {
                    textBoxInfo.AppendText("Reset Device " + ctrSubdevice.deviceIndex.ToString() + ", Subdevice " + ctrSubdevice.subdevIndex.ToString() + Environment.NewLine);
                } 
                else
                {
                    DisplayErrorMessage("meIOResetSubdevice - Error: ", meError);             
                }
            }
        }

        private void btnStop1_Click(object sender, EventArgs e)
        {
            StopTimer1();
        }

        private void btnStop2_Click(object sender, EventArgs e)
        {
            StopTimer2();
        }

        private void btnStop3_Click(object sender, EventArgs e)
        {
            StopTimer3();
        }

        private void StopTimer(int ctrIdx)
        {
            switch (ctrIdx)
            {
                case 0:
                    StopTimer1();
                    break;
                case 1:
                    StopTimer2();
                    break;
                case 2:
                    StopTimer3();
                    break;
                default:
                    StopAllTimers();
                    break;
            }
        }

        private void StopTimer1()
        {
            tmr1.Enabled = false;
            btnRun1.Enabled = true;
        }

        private void StopTimer2()
        {
            tmr2.Enabled = false;
            btnRun2.Enabled = true;
        }

        private void StopTimer3()
        {
            tmr3.Enabled = false;
            btnRun3.Enabled = true;
        }

        private void StopAllTimers()
        {
            StopTimer1();
            StopTimer2();
            StopTimer3();
        }

        private void frmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            CloseMEiDSDriver();
        }

        private void CloseMEiDSDriver()
        {
            StopAllTimers();
            if (openErrSuccess == meIDS.ME_ERRNO_SUCCESS)
            {
                meIDS.meClose(meIDS.ME_CLOSE_NO_FLAGS);         
                openErrSuccess = Constants.ME_IDS_NOTOPENED;
                subdeviceList.Clear();
                coBoxSubDevs.Items.Clear();
            }
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            Close();
        }

        private string MeVersionToString(int version)
        {
            return String.Format("{0}.{1}.{2}.{3}", version >> 24, 0xFF & (version >> 16), 0xFF & (version >> 8), 0xFF & version);
        }

        private void DisplayErrorMessage(string message, int meError)
        {
            StringBuilder errMessage = new StringBuilder(meIDS.ME_ERROR_MSG_MAX_COUNT);    
            textBoxInfo.AppendText(message + meError.ToString() + Environment.NewLine);
            meIDS.meErrorGetMessage(meError, errMessage, errMessage.Capacity);             
            textBoxInfo.AppendText("ErrorMessage: " + errMessage + Environment.NewLine);   
        }

        private void btnClearHwInfo_Click(object sender, EventArgs e)
        {
            textBoxInfo.Clear();
            lineNumber = 0;
        }

        private void btnAbout_Click(object sender, EventArgs e)
        {
            AboutBox1 aboutBox = new AboutBox1();
            aboutBox.ShowDialog();
        }

        private void coBoxClockSource1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (tmr1.Enabled) StopTimer1();
        }

        private void coBoxClockSource2_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (tmr2.Enabled) StopTimer2();
        }

        private void coBoxClockSource3_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (tmr3.Enabled) StopTimer3();
        }

        private void coBoxMode1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (tmr1.Enabled) StopTimer1();
        }

        private void coBoxMode2_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (tmr2.Enabled) StopTimer2();
        }

        private void coBoxMode3_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (tmr3.Enabled) StopTimer3();
        }

        private void btnPreload1_Click(object sender, EventArgs e)
        {
            StopTimer1();
            SingleWriteCounter(0);
        }

        private void btnPreload2_Click(object sender, EventArgs e)
        {
            StopTimer2();
            SingleWriteCounter(1);
        }

        private void btnPreload3_Click(object sender, EventArgs e)
        {
            StopTimer3();
            SingleWriteCounter(2);
        }

        private void tmr1_Tick(object sender, EventArgs e)
        {
            int meError = SingleReadCounter(0, editCtrValue1);
            if (meError != meIDS.ME_ERRNO_SUCCESS && meError != meIDS.ME_ERRNO_SUBDEVICE_NOT_RUNNING)
            {
                StopTimer1();
                DisplayErrorMessage(grpBoxCounter1.Text + " meIOSingle Error: ", meError);
            }
        }

        private void tmr2_Tick(object sender, EventArgs e)
        {
            int meError = SingleReadCounter(1, editCtrValue2);
            if (meError != meIDS.ME_ERRNO_SUCCESS && meError != meIDS.ME_ERRNO_SUBDEVICE_NOT_RUNNING)
            {
                StopTimer2();
                DisplayErrorMessage(grpBoxCounter2.Text + " meIOSingle Error: ", meError);
            }
        }

        private void tmr3_Tick(object sender, EventArgs e)
        {
            int meError = SingleReadCounter(2, editCtrValue3);
            if (meError != meIDS.ME_ERRNO_SUCCESS && meError != meIDS.ME_ERRNO_SUBDEVICE_NOT_RUNNING)
            {
                StopTimer3();
                DisplayErrorMessage(grpBoxCounter3.Text + " meIOSingle Error: ", meError);
            }
        }

        private void btnRun1_Click(object sender, EventArgs e)
        {
            btnRun1.Enabled = false;
            if (SingleWriteCounter(0))
            {
                tmr1.Enabled = true;
            } 
            else
            {
                btnRun1.Enabled = true;
            }
        }

        private void btnRun2_Click(object sender, EventArgs e)
        {
            btnRun2.Enabled = false;
            if (SingleWriteCounter(1))
            {
                tmr2.Enabled = true;
            }
            else
            {
                btnRun2.Enabled = true;
            }
        }

        private void btnRun3_Click(object sender, EventArgs e)
        {
            btnRun3.Enabled = false;
            if (SingleWriteCounter(2))
            {
                tmr3.Enabled = true;
            }
            else
            {
                btnRun3.Enabled = true;
            }
        }

        private void btnRead1_Click(object sender, EventArgs e)
        {
            int meError = SingleReadCounter(0, editCtrValue1);
            if (meError != meIDS.ME_ERRNO_SUCCESS)
            {
                DisplayErrorMessage(grpBoxCounter1.Text + " meIOSingle Error: ", meError);
            }
        }

        private void btnRead2_Click(object sender, EventArgs e)
        {
            int meError = SingleReadCounter(1, editCtrValue2);
            if (meError != meIDS.ME_ERRNO_SUCCESS)
            {
                DisplayErrorMessage(grpBoxCounter2.Text + " meIOSingle Error: ", meError);
            }
        }

        private void btnRead3_Click(object sender, EventArgs e)
        {
            int meError = SingleReadCounter(2, editCtrValue3);
            if (meError != meIDS.ME_ERRNO_SUCCESS)
            {
                DisplayErrorMessage(grpBoxCounter3.Text + " meIOSingle Error: ", meError);
            }
        }

        private void btnReset1_Click(object sender, EventArgs e)
        {
            ResetSubdevice(0);
            editCtrValue1.Text = "0";
        }

        private void btnReset2_Click(object sender, EventArgs e)
        {
            ResetSubdevice(1);
            editCtrValue2.Text = "0";
        }

        private void btnReset3_Click(object sender, EventArgs e)
        {
            ResetSubdevice(2);
            editCtrValue3.Text = "0";
        }

        private void DisplaySubdevices()
        {
            if (coBoxSubDevs.Items.Count > 0)
            {
                textBoxInfo.AppendText(String.Format("Number of Counter sub-devices found: {0}", coBoxSubDevs.Items.Count * 3) + Environment.NewLine);
                for (int i = 0; i < coBoxSubDevs.Items.Count; i++)
                {
                    textBoxInfo.AppendText(coBoxSubDevs.Items[i] + Environment.NewLine);  
                }
                textBoxInfo.AppendText(Environment.NewLine);
                coBoxSubDevs.SelectedIndex = 0;                             
                ChangeCounterSubdevices(coBoxSubDevs.SelectedIndex);
            }
            else
            {
                textBoxInfo.AppendText("No Counter sub-devices of subtype 8254 found." + Environment.NewLine);  
                CloseMEiDSDriver();
            }
        }

        private int GetDevicePlugged(int idxDevice, out int plugged)
        {
            int vendorId;
            int deviceId;
            int serialNo;
            int busType;
            int busNo;
            int devNo;
            int funcNo;

            int meError = meIDS.meQueryInfoDevice(idxDevice,
                                             out vendorId,
                                             out deviceId,
                                             out serialNo,
                                             out busType,
                                             out busNo,
                                             out devNo,
                                             out funcNo,
                                             out plugged);
            return meError;
        }

        private void DisplayVersionMainDriver()
        {
            int tmpVersion;
            meIDS.meQueryVersionMainDriver(out tmpVersion);
            textBoxInfo.AppendText("Maindriver version: " + MeVersionToString(tmpVersion) + Environment.NewLine);
        }

        private int DisplayVersionLibrary()
        {
            int tmpVersion;
            meIDS.meQueryVersionLibrary(out tmpVersion);
            textBoxInfo.AppendText("Library version: " + MeVersionToString(tmpVersion) + Environment.NewLine);
            return tmpVersion;
        }

        private void AddCoBoxValue(ComboBox coBox, string newStr)
        {
            string tmpStr = newStr.Trim();
            if (tmpStr.Length > 0 && coBox.Items.IndexOf(tmpStr) < 0)
            {
                coBox.Items.Add(tmpStr);
            }
        }
    }

    class CounterSubdevice
    {
        public string deviceName;
        public int deviceIndex;            
        public int subdevIndex;            
        public int subdevCaps;             
        public int counterMaxValue;        
        public CounterSubdevice(string DeviceName, int DeviceIndex, int SubdevIndex, int Capabilities, int CounterMaxValue)
        {
            this.deviceName = DeviceName;
            this.deviceIndex = DeviceIndex;
            this.subdevIndex = SubdevIndex;
            this.subdevCaps = Capabilities;
            this.counterMaxValue = CounterMaxValue;
        }
    }

    public class ComboBoxItem
    {
        string dispValue;
        int val;

        public ComboBoxItem(string displayValue, int value)
        {
            dispValue = displayValue;
            val = value;
        }

        public override string ToString()
        {
            return dispValue;
        }

        public int GetValue
        {
            get
            {
                return val;
            }
        }
    }

    static class Constants
    {
        public const int ME_IDS_NOTOPENED = -1;           
        public static int[] Ctr8254Modes = {meIDS.ME_SINGLE_CONFIG_CTR_8254_MODE_0,
                                            meIDS.ME_SINGLE_CONFIG_CTR_8254_MODE_1,
                                            meIDS.ME_SINGLE_CONFIG_CTR_8254_MODE_2,
                                            meIDS.ME_SINGLE_CONFIG_CTR_8254_MODE_3,
                                            meIDS.ME_SINGLE_CONFIG_CTR_8254_MODE_4,
                                            meIDS.ME_SINGLE_CONFIG_CTR_8254_MODE_5}; 

    }
}
