﻿//==============================================================================
//Copyright(c), Meilhaus Electronic GmbH
//              http://www.meilhaus.de
//              support@meilhaus.de
//
//Source File : FormMain.cs
//Description : Example program for writing single values to Analog-out (AO) subdevices
//              - Open ME-iDS main driver when program is started
//              - Go through the list of all devices and find AO subdevices
//              - Find AO subdevices and information about channels and ranges
//              - Add AO subdevices to the combobox, so the user can select one
//              - Write data to selected subdevice when user clicks on button "Write Data"
//              - Close ME-iDS main driver when program is closed
//==============================================================================
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using meIDS_CS;

namespace Single_Write_Analog
{
    public partial class frmMain : Form
    {
        private int openErrSuccess = Constants.ME_IDS_NOTOPENED;
        private int dataLine = 0;
        private List<AoSubdevice> subdeviceList = new List<AoSubdevice>();      

        public frmMain()
        {
            InitializeComponent();
        }

        private void frmMain_Shown(object sender, EventArgs e)
        {
            OpenMEiDSDriver();
        }

        private void frmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            CloseMEiDSDriver();
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void OpenMEiDSDriver()
        {
            try
            {
                textBoxInfo.AppendText("Opening Maindriver..." + Environment.NewLine);
                openErrSuccess = meIDS.meOpen(meIDS.ME_OPEN_NO_FLAGS);          

                if (openErrSuccess == meIDS.ME_ERRNO_SUCCESS)
                {
                    DisplayVersionLibrary();
                    DisplayVersionMainDriver();
                    textBoxInfo.AppendText(Environment.NewLine);

                    int numDevices;
                    meIDS.meQueryNumberDevices(out numDevices);                     
                    textBoxInfo.AppendText("Number of devices found: " + numDevices.ToString() + Environment.NewLine);

                    GetAoSubdevices(numDevices);
                }
                else
                {
                    DisplayErrorMessage("meOpen - Error: ", openErrSuccess);
                }
            }
            catch (DllNotFoundException ex)
            {
                textBoxInfo.AppendText("The ME-iDS is not properly installed." + Environment.NewLine);
                textBoxInfo.AppendText(ex.Message + Environment.NewLine);
            }
        }

        private void DisplayVersionMainDriver()
        {
            int tmpVersion;
            meIDS.meQueryVersionMainDriver(out tmpVersion);
            textBoxInfo.AppendText("Maindriver version: " + MeVersionToString(tmpVersion) + Environment.NewLine);
        }

        private void DisplayVersionLibrary()
        {
            int tmpVersion;
            meIDS.meQueryVersionLibrary(out tmpVersion);
            textBoxInfo.AppendText("Library version: " + MeVersionToString(tmpVersion) + Environment.NewLine);            
        }

        private void GetAoSubdevices(int numDevices)
        {
            if (openErrSuccess == meIDS.ME_ERRNO_SUCCESS)
            {
                textBoxInfo.AppendText("Retrieving subdevices of type ME_TYPE_AO (Analog-Out)" + Environment.NewLine);
                subdeviceList.Clear();
                coBoxSubDevs.Items.Clear();
                
                for (int idxDevice = 0; idxDevice < numDevices; idxDevice++)
                {
                    int plugged;
                    int meError = GetDevicePlugged(idxDevice, out plugged);

                    if (meError == meIDS.ME_ERRNO_SUCCESS && plugged == meIDS.ME_PLUGGED_IN)
                    {
                        GetSubdevicesByType(idxDevice, meIDS.ME_TYPE_AO); 
                    }
                    else
                    {
                        textBoxInfo.AppendText("Device " + idxDevice.ToString() + " not plugged in" + Environment.NewLine);
                    }
                }
            }
            DisplaySubdevices();
        }

        private void DisplaySubdevices()
        {
            if (coBoxSubDevs.Items.Count > 0)
            {
                textBoxInfo.AppendText("Number of Analog-Out sub-devices found: " + coBoxSubDevs.Items.Count.ToString() + Environment.NewLine);
                for (int i = 0; i < coBoxSubDevs.Items.Count; i++)
                {
                    textBoxInfo.AppendText(coBoxSubDevs.Items[i] + Environment.NewLine); 
                }
                textBoxInfo.AppendText(Environment.NewLine);
                coBoxSubDevs.SelectedIndex = 0;                             
                btnSingleWrite.Enabled = true;
                btnResetSubdev.Enabled = true;
            }
            else
            {
                textBoxInfo.AppendText("No Analog-Out sub-devices found." + Environment.NewLine);
                CloseMEiDSDriver();
            }
        }

        private void GetSubdevicesByType(int idxDevice, int subdeviceType)
        {
            int meError;
            int numSubdevices;
            int idxMatchedSubdevice;
            StringBuilder devName = new StringBuilder(meIDS.ME_DEVICE_NAME_MAX_COUNT);
            
            meError = meIDS.meQueryNumberSubdevices(idxDevice, out numSubdevices);  
            if (meError == meIDS.ME_ERRNO_SUCCESS)
            {
                int idxSubdevice = 0;
                meIDS.meQueryNameDevice(idxDevice, devName, devName.Capacity);  
                while (idxSubdevice < numSubdevices)
                {
                    meError = meIDS.meQuerySubdeviceByType(idxDevice,                 
                                                           idxSubdevice,              
                                                           subdeviceType,             
                                                           meIDS.ME_SUBTYPE_ANY,      
                                                           out idxMatchedSubdevice);  

                    if (meError == meIDS.ME_ERRNO_SUCCESS)
                    {
                        meError = AddAoSubdevice(devName.ToString(), idxDevice, idxMatchedSubdevice, subdeviceType);
                        idxSubdevice = idxMatchedSubdevice + 1;                 
                    }
                    else
                    {
                        break;                                                  
                    }
                }
            }
        }

        private int AddAoSubdevice(String devName, int idxDevice, int idxSubdevice, int subdeviceType)
        {
            int numOfChannels;
            int meError = meIDS.meQueryNumberChannels(idxDevice, idxSubdevice, out numOfChannels);  
            if (meError == meIDS.ME_ERRNO_SUCCESS)
            {
                AoSubdevice aoSubdev = new AoSubdevice(devName, idxDevice, idxSubdevice, numOfChannels);
                subdeviceList.Add(aoSubdev);
                AddAnalogRanges(aoSubdev);

                coBoxSubDevs.Items.Add(devName.ToString()
                                        + ",  Device " + idxDevice.ToString()
                                        + ",  Subdevice " + idxSubdevice.ToString()
                                        + ",  Type: " + GetSubDevTypeName(subdeviceType)
                                        + ",  Channels: " + numOfChannels.ToString()
                                        + ",  Ranges: " + aoSubdev.GetRangeCount().ToString());
            }
            return meError;
        }

        private void AddAnalogRanges(AoSubdevice aoSubdevice)
        {
            int idxRange;
            int numOfRanges;
            meIDS.meQueryNumberRanges(aoSubdevice.deviceIndex,
                                      aoSubdevice.subdevIndex,  
                                      meIDS.ME_UNIT_ANY,    
                                      out numOfRanges);     
            idxRange = 0;

            while (idxRange < numOfRanges)
            {
                AddAnalogRange(aoSubdevice, idxRange);
                idxRange++;
            };
        }

        private void AddAnalogRange(AoSubdevice aoSubdevice, int idxRange)
        {
            int unit;
            Double physMin;
            Double physMax;
            int digitalMax;
            
            int meError = meIDS.meQueryRangeInfo(aoSubdevice.deviceIndex,
                      aoSubdevice.subdevIndex,
                      idxRange,
                      out unit,
                      out physMin,
                      out physMax,
                      out digitalMax);
            if (meError == meIDS.ME_ERRNO_SUCCESS)
            {
                aoSubdevice.AddRange(unit, digitalMax, physMin, physMax);
            }
        }

        private void CloseMEiDSDriver()
        {
            if (openErrSuccess == meIDS.ME_ERRNO_SUCCESS)
            {
                meIDS.meClose(meIDS.ME_CLOSE_NO_FLAGS);                         
                openErrSuccess = Constants.ME_IDS_NOTOPENED;
                btnSingleWrite.Enabled = false;
                btnResetSubdev.Enabled = false;
                subdeviceList.Clear();
                coBoxChannel.Items.Clear();
                coBoxSubDevs.Items.Clear();
                coBoxRange.Items.Clear();
            }
        }

        private void coBoxSubDevs_SelectedIndexChanged(object sender, EventArgs e)
        {
            FillSubdeviceProperties(coBoxSubDevs.SelectedIndex);
        }

        private void FillSubdeviceProperties(int idxSelectedSubdev)
        {            
            if (idxSelectedSubdev >= 0)
            {
                AoSubdevice aoSubDevice = subdeviceList[idxSelectedSubdev];
                FillSubdeviceChannels(aoSubDevice);
                FillSubdeviceRanges(aoSubDevice);
                InitTrackbarPosition(coBoxRange.SelectedIndex);
            }
        }

        private void FillSubdeviceRanges(AoSubdevice aoSubDevice)
        {
            AnalogRange aoRange;
            coBoxRange.Items.Clear();
            for (int i = 0; i < aoSubDevice.GetRangeCount(); i++)
            {
                aoRange = aoSubDevice.GetRange(i);
                coBoxRange.Items.Add("Range " + i.ToString()
                                    + ": " + aoSubDevice.GetRange(i).displayRange
                 );
            }
            if (coBoxRange.Items.Count > 0) coBoxRange.SelectedIndex = 0;
        }

        private void FillSubdeviceChannels(AoSubdevice aoSubDevice)
        {
            coBoxChannel.Items.Clear();
            for (int i = 0; i < aoSubDevice.numOfChannels; i++)
            {
                coBoxChannel.Items.Add("Channel " + i.ToString());
            };
            if (coBoxChannel.Items.Count > 0) coBoxChannel.SelectedIndex = 0;
        }

        private void coBoxRange_SelectedIndexChanged(object sender, EventArgs e)
        {
            InitTrackbarPosition(coBoxRange.SelectedIndex);
        }

        private void InitTrackbarPosition(int idxSelectedRange)
        {
            if (idxSelectedRange >= 0)
            {
                AoSubdevice aoSubDevice = subdeviceList[coBoxSubDevs.SelectedIndex]; 
                AnalogRange aoRange = aoSubDevice.GetRange(idxSelectedRange);   

                trackbarVal.SetRange(0, aoRange.digitalMax);
                SetTrackPosition(0, aoRange);
                trackbarVal.TickFrequency = trackbarVal.Maximum / 20;

                txtBoxPhysValue.Text = "0";
            }
        }

        private void txtBoxPhysValue_KeyDown(object sender, KeyEventArgs e)
        {
            if ((e.KeyCode == Keys.Return) && (coBoxSubDevs.SelectedIndex >= 0))
            {
                Double valAnalog;
                AoSubdevice aoSubDevice = subdeviceList[coBoxSubDevs.SelectedIndex]; 
                AnalogRange aoRange = aoSubDevice.GetRange(coBoxRange.SelectedIndex); 
                try
                {
                    valAnalog = CheckValueMinMax(double.Parse(txtBoxPhysValue.Text), aoRange);
                    SetTrackPosition(valAnalog, aoRange);
                }
                catch (System.Exception ex)
                {
                    MessageBox.Show("Error: " + ex.Message);
                    txtBoxPhysValue.Focus();
                }
            }
        }

        private void SetTrackPosition(Double valAnalog, AnalogRange aoRange)
        {
            int trckVal = (int)((valAnalog - aoRange.physicalMin) * aoRange.digitalMax / (aoRange.physicalMax - aoRange.physicalMin));
            trackbarVal.Value = trckVal;
        }

        private void trackbarVal_ValueChanged(object sender, EventArgs e)
        {
            if (coBoxSubDevs.Items.Count > 0)
            {
                Double valAnalog;
                AoSubdevice aoSubDevice = subdeviceList[coBoxSubDevs.SelectedIndex]; 
                AnalogRange aoRange = aoSubDevice.GetRange(coBoxRange.SelectedIndex); 
                Double physRange = aoRange.physicalMax - aoRange.physicalMin;
                valAnalog = aoRange.physicalMin + (physRange / (trackbarVal.Maximum - trackbarVal.Minimum)) * trackbarVal.Value;
                txtBoxPhysValue.Text = String.Format("{0:0.000}", valAnalog);
            }
        }

        private void btnSingleWrite_Click(object sender, EventArgs e)
        {
            if (coBoxSubDevs.SelectedIndex >= 0)
            {
                AoSubdevice aoSubDevice = subdeviceList[coBoxSubDevs.SelectedIndex];
                AnalogRange aoRange = aoSubDevice.GetRange(coBoxRange.SelectedIndex);
                
                try
                {
                    Double valAnalog;
                    valAnalog = CheckValueMinMax(double.Parse(txtBoxPhysValue.Text), aoRange);
                    SetTrackPosition(valAnalog, aoRange);
                    WriteToSubdevice(aoSubDevice, aoRange, valAnalog);
                }
                catch (System.Exception ex)
                {
                    MessageBox.Show("Error: " + ex.Message);
                }
            }
        }

        private void WriteToSubdevice(AoSubdevice aoSubDevice, AnalogRange aoRange, Double valAnalog)
        {
            int meError;
            int valDigital;
            meError = ConvertPhysicalToDigital(aoSubDevice, aoRange, valAnalog, out valDigital);

            if (meError == meIDS.ME_ERRNO_SUCCESS)
            {
                meError = ConfigureSubdeviceForOutput(aoSubDevice);
                if (meError == meIDS.ME_ERRNO_SUCCESS)
                {
                    meError = WriteSingleValue(aoSubDevice, valDigital);
                    if (meError == meIDS.ME_ERRNO_SUCCESS)
                    {
                        DisplayValue(aoSubDevice, coBoxChannel.SelectedIndex, valAnalog, valDigital);
                    }
                }
                if (meError != meIDS.ME_ERRNO_SUCCESS)
                {
                    DisplayErrorMessage("meIOSingle - Error: ", meError);
                }
            }
            else
            {
                DisplayErrorMessage("meUtilityPhysicalToDigital - Error: ", meError);
            }
        }

        private int ConvertPhysicalToDigital(AoSubdevice aoSubDevice, AnalogRange aoRange, Double valAnalog, out int valDigital)
        {
            int meError = meIDS.meUtilityPhysicalToDigital(aoRange.physicalMin, 
                                                    aoRange.physicalMax,  
                                                    aoRange.digitalMax,   
                                                    valAnalog,            
                                                    out valDigital);      
            return meError;
        }

        private Double CheckValueMinMax(Double valAnalog, AnalogRange aoRange)
        {
            if (valAnalog < aoRange.physicalMin)
            {
                valAnalog = aoRange.physicalMin;
                txtBoxPhysValue.Text = String.Format("{0:0.000}", valAnalog);
            }
            else if (valAnalog > aoRange.physicalMax)
            {
                valAnalog = aoRange.physicalMax;
                txtBoxPhysValue.Text = String.Format("{0:0.000}", valAnalog);
            }
            return valAnalog;
        }

        private int ConfigureSubdeviceForOutput(AoSubdevice aoSubDevice)
        {
            int meError = meIDS.meIOSingleConfig(aoSubDevice.deviceIndex,  
                                             aoSubDevice.subdevIndex,  
                                             coBoxChannel.SelectedIndex, 
                                             coBoxRange.SelectedIndex, 
                                             meIDS.ME_REF_AO_GROUND,   
                                             meIDS.ME_TRIG_CHAN_DEFAULT, 
                                             meIDS.ME_TRIG_TYPE_SW,	
                                             meIDS.ME_VALUE_NOT_USED, 
                                             meIDS.ME_IO_SINGLE_CONFIG_NO_FLAGS); 
            return meError;
        }

        private int WriteSingleValue(AoSubdevice aoSubDevice, int valDigital)
        {
            meIDS.meIOSingle_t[] io_single = new meIDS.meIOSingle_t[1]; 
            
            io_single[0].iDevice = aoSubDevice.deviceIndex;             
            io_single[0].iSubdevice = aoSubDevice.subdevIndex;          
            io_single[0].iChannel = coBoxChannel.SelectedIndex;         
            io_single[0].iDir = meIDS.ME_DIR_OUTPUT;                    
            io_single[0].iValue = valDigital;                           
            io_single[0].iTimeOut = 0;                                  
            io_single[0].iFlags = meIDS.ME_IO_SINGLE_TYPE_NO_FLAGS;     
            io_single[0].iErrno = 0;                                    

            int meError = meIDS.meIOSingle(ref io_single[0],               
                                       1,                               
                                       meIDS.ME_IO_SINGLE_NO_FLAGS);    
            return meError;
        }

        private void DisplayValue(AoSubdevice aoSubDevice, int idxRange, Double valAnalog, int valDigital)
        {
            dataLine++;
            textBoxInfo.AppendText(String.Format("{0:00}", dataLine)     
                        + " Write to " + aoSubDevice.deviceName
                        + ",  Device: " + aoSubDevice.deviceIndex.ToString()
                        + ",  Subdevice: " + aoSubDevice.subdevIndex.ToString()
                        + ",  Digital value: " + valDigital.ToString()
                        + ",  Physical value: " + String.Format("{0:0.000}", valAnalog)    
                        + " " + aoSubDevice.GetRange(idxRange).displayUnit       
                        + Environment.NewLine);
        }

        private void btnResetSubdev_Click(object sender, EventArgs e)
        {
            AoSubdevice aoSubdev = subdeviceList[coBoxSubDevs.SelectedIndex];
            int meError = meIDS.meIOResetSubdevice(aoSubdev.deviceIndex,
                                                  aoSubdev.subdevIndex,
                                                  meIDS.ME_IO_RESET_SUBDEVICE_NO_FLAGS);
            if (meError == meIDS.ME_ERRNO_SUCCESS)
            {
                textBoxInfo.AppendText("Reset Device " + aoSubdev.deviceIndex.ToString() + ", Subdevice " + aoSubdev.subdevIndex.ToString() + Environment.NewLine);
            }
            else
            {
                DisplayErrorMessage("Error reset Subdevice", meError);
            }
        }

        private void DisplayErrorMessage(string message, int meError)
        {
            StringBuilder errMessage = new StringBuilder(meIDS.ME_ERROR_MSG_MAX_COUNT);
            textBoxInfo.AppendText(message + meError.ToString() + Environment.NewLine);
            meIDS.meErrorGetMessage(meError, errMessage, errMessage.Capacity);             
            textBoxInfo.AppendText("ErrorMessage: " + errMessage + Environment.NewLine);    
        }

        private int GetDevicePlugged(int idxDevice, out int plugged)
        {
            int vendorId;
            int deviceId;
            int serialNo;
            int busType;
            int busNo;
            int devNo;
            int funcNo;

            int meError = meIDS.meQueryInfoDevice(idxDevice,
                                             out vendorId,
                                             out deviceId,
                                             out serialNo,
                                             out busType,
                                             out busNo,
                                             out devNo,
                                             out funcNo,
                                             out plugged);              //Indicates whether a device is physically available
            return meError;
        }

        private string GetSubDevTypeName(int subdeviceType)
        {
            string tmpType;
            switch (subdeviceType)
            {
                case meIDS.ME_TYPE_INVALID:
                    tmpType = "ME_TYPE_INVALID";
                    break;
                case meIDS.ME_TYPE_AI:
                    tmpType = "ME_TYPE_AI";
                    break;
                case meIDS.ME_TYPE_AO:
                    tmpType = "ME_TYPE_AO";
                    break;
                case meIDS.ME_TYPE_DIO:
                    tmpType = "ME_TYPE_DIO";
                    break;
                case meIDS.ME_TYPE_DO:
                    tmpType = "ME_TYPE_DO";
                    break;
                case meIDS.ME_TYPE_DI:
                    tmpType = "ME_TYPE_DI";
                    break;
                case meIDS.ME_TYPE_CTR:
                    tmpType = "ME_TYPE_CTR";
                    break;
                case meIDS.ME_TYPE_EXT_IRQ:
                    tmpType = "ME_TYPE_EXT_IRQ";
                    break;
                case meIDS.ME_TYPE_FIO:
                    tmpType = "ME_TYPE_FIO";
                    break;
                case meIDS.ME_TYPE_FO:
                    tmpType = "ME_TYPE_FO";
                    break;
                case meIDS.ME_TYPE_FI:
                    tmpType = "ME_TYPE_FI";
                    break;
                case meIDS.ME_TYPE_FPGA:
                    tmpType = "ME_TYPE_FPGA";
                    break;
                default:
                    tmpType = String.Format("0x{0:X}", subdeviceType);
                    break;
            };
            return tmpType;
        }

        private string MeVersionToString(int version)
        {
            return String.Format("{0}.{1}.{2}.{3}", version >> 24, 0xFF & (version >> 16), 0xFF & (version >> 8), 0xFF & version);
        }

        private void btnAbout_Click(object sender, EventArgs e)
        {
            AboutBox1 aboutBox = new AboutBox1();
            aboutBox.ShowDialog();
        }

        private void btnClearHwInfo_Click(object sender, EventArgs e)
        {
            textBoxInfo.Clear();
            dataLine = 0;
        }
    }

    class AoSubdevice
    {
        public string deviceName;
        public int deviceIndex;
        public int subdevIndex;
        public int numOfChannels;
        private List<AnalogRange> analogRanges = new List<AnalogRange>();                
        
        public AoSubdevice(string DeviceName, int DeviceIndex, int SubdevIndex, int NumOfChannels)
        {
            this.deviceName = DeviceName;
            this.deviceIndex = DeviceIndex;
            this.subdevIndex = SubdevIndex;
            this.numOfChannels = NumOfChannels;
        }
        
        public void AddRange(int RangeUnit, int DigitalMax, Double PhysicalMin, Double PhysicalMax)
        {
            AnalogRange aoRng = new AnalogRange(RangeUnit, DigitalMax, PhysicalMin, PhysicalMax);
            analogRanges.Add(aoRng);
        }
        
        public AnalogRange GetRange(int index)
        {
            return analogRanges[index];
        }

        public int GetRangeCount()
        {
            return analogRanges.Count;
        }
    }

    class AnalogRange
    {
        public int rangeUnit;
        public int digitalMax;
        public Double physicalMin;
        public Double physicalMax;
        public string displayRange;
        public string displayUnit;
        public AnalogRange(int RangeUnit, int DigitalMax, Double PhysicalMin, Double PhysicalMax)
        {
            this.rangeUnit = RangeUnit;
            this.digitalMax = DigitalMax;
            this.physicalMin = PhysicalMin;
            this.physicalMax = PhysicalMax;
            this.SetRangeUnitTexts();
        }

        private void SetRangeUnitTexts()
        {
            Double tmpVal = Math.Abs(physicalMin);
            StringBuilder sbRange = new StringBuilder();

            if (tmpVal == 0)
            {
                sbRange.Append("0");
            } 
            else if (tmpVal < 0.001)
            {
                sbRange.Append(string.Format("{0:0.00}", physicalMin * 1000000));
                sbRange.Append("µ");
            } 
            else if (tmpVal < 1)
            {
                sbRange.Append(string.Format("{0:0.00}", physicalMin * 1000));
                sbRange.Append("m");
            } 
            else
            {
                sbRange.Append(string.Format("{0:0.00}", Math.Round(physicalMin, 2)));

            };

            tmpVal = Math.Abs(physicalMax);
            if (tmpVal == 0)
            {
                sbRange.Append("...0");
            } 
            else if (tmpVal < 0.001)
            {
                sbRange.Append(string.Format("...{0:0.00}", physicalMax * 1000000));
                sbRange.Append("µ");
            } 
            else if (tmpVal < 1)
            {
                sbRange.Append(string.Format("...{0:0.00}", physicalMax * 1000));
                sbRange.Append("m");
            } 
            else
            {
                sbRange.Append(string.Format("...{0:0.00}", Math.Round(physicalMax, 2)));
            }

            if (rangeUnit == meIDS.ME_UNIT_VOLT)
            {
                displayUnit = "Volt";
                sbRange.Append("V");
            }
            else if (rangeUnit == meIDS.ME_UNIT_AMPERE)
            {
                displayUnit = "Ampere";
                sbRange.Append("A");
            }

            displayRange = sbRange.ToString();  //gives e.g. -10...10V
        }
    }
    
    static class Constants
    {
        public const int ME_IDS_NOTOPENED = -1;
    }
}
